@@ -1,21 +1,21 @@
-/*      $Id: display.c 22182 2006-06-22 19:49:53Z olivier $
- 
+/*      $Id: display.c 23704 2006-11-04 22:36:42Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
- 
+
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,26 +26,43 @@
 #include <X11/Xutil.h>
 #include <X11/cursorfont.h>
 #include <X11/extensions/shape.h>
-#include <gtk/gtk.h>
 #include <glib.h>
-#include <libxfce4util/libxfce4util.h> 
-#include <libxfcegui4/libxfcegui4.h>
+#include <gdk/gdk.h>
+#include <gdk/gdkx.h>
+#include <gtk/gtk.h>
+#include <libxfce4util/libxfce4util.h>
+#ifdef HAVE_RENDER
+#include <X11/extensions/Xrender.h>
+#endif
+
 #include "spinning_cursor.h"
 #include "display.h"
 #include "screen.h"
 #include "client.h"
 #include "compositor.h"
 
+#ifndef MAX_HOSTNAME_LENGTH
+#define MAX_HOSTNAME_LENGTH 32
+#endif /* MAX_HOSTNAME_LENGTH */
+
+#ifndef CURSOR_ROOT
+#define CURSOR_ROOT XC_left_ptr
+#endif
+
+#ifndef CURSOR_MOVE
+#define CURSOR_MOVE XC_fleur
+#endif
+
 static int
 handleXError (Display * dpy, XErrorEvent * err)
 {
-#if DEBUG            
+#if DEBUG
     char buf[64];
 
     XGetErrorText (dpy, err->error_code, buf, 63);
-    g_print ("XError: %s\n", buf);                                                  
-    g_print ("==>  XID Ox%lx, Request %d, Error %d <==\n", 
-              err->resourceid, err->request_code, err->error_code); 
+    g_print ("XError: %s\n", buf);                                    
+    g_print ("==>  XID 0x%lx, Request %d, Error %d <==\n",
+              err->resourceid, err->request_code, err->error_code);
 #endif
     return 0;
 }
@@ -53,7 +70,8 @@ handleXError (Display * dpy, XErrorEvent * err)
 static gboolean
 myDisplayInitAtoms (DisplayInfo *display_info)
 {
-    char *atom_names[] = {
+    static const char *atom_names[] = {
+        "COMPOSITING_MANAGER",
         "GNOME_PANEL_DESKTOP_AREA",
         "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR",
         "KWM_WIN_ICON",
@@ -93,6 +111,7 @@ myDisplayInitAtoms (DisplayInfo *display_info)
         "_NET_WM_MOVERESIZE",
         "_NET_WM_NAME",
         "_NET_WM_WINDOW_OPACITY",
+        "_NET_WM_WINDOW_OPACITY_LOCKED",
         "_NET_WM_STATE",
         "_NET_WM_STATE_ABOVE",
         "_NET_WM_STATE_BELOW",
@@ -108,6 +127,8 @@ myDisplayInitAtoms (DisplayInfo *display_info)
         "_NET_WM_STATE_STICKY",
         "_NET_WM_STRUT",
         "_NET_WM_STRUT_PARTIAL",
+        "_NET_WM_SYNC_REQUEST",
+        "_NET_WM_SYNC_REQUEST_COUNTER",
         "_NET_WM_USER_TIME",
         "_NET_WM_WINDOW_TYPE",
         "_NET_WM_WINDOW_TYPE_DESKTOP",
@@ -134,6 +155,7 @@ myDisplayInitAtoms (DisplayInfo *display_info)
         "_WIN_WORKSPACE_COUNT",
         "WM_CHANGE_STATE",
         "WM_CLIENT_LEADER",
+        "WM_CLIENT_MACHINE",
         "WM_COLORMAP_WINDOWS",
         "WM_DELETE_WINDOW",
         "WM_HINTS",
@@ -142,14 +164,16 @@ myDisplayInitAtoms (DisplayInfo *display_info)
         "WM_TAKE_FOCUS",
         "WM_TRANSIENT_FOR",
         "WM_WINDOW_ROLE",
+        "XFWM4_COMPOSITING_MANAGER",
+        "XFWM4_TIMESTAMP_PROP",
         "_XROOTPMAP_ID",
         "_XSETROOT_ID"
     };
-    
-    g_assert (NB_ATOMS == G_N_ELEMENTS (atom_names));
-    return (XInternAtoms (display_info->dpy, 
-                          atom_names, 
-                          NB_ATOMS,
+
+    g_assert (ATOM_COUNT == G_N_ELEMENTS (atom_names));
+    return (XInternAtoms (display_info->dpy,
+                          (char **) atom_names,
+                          ATOM_COUNT,
                           FALSE, display_info->atoms) != 0);
 }
 
@@ -157,6 +181,9 @@ DisplayInfo *
 myDisplayInit (GdkDisplay *gdisplay)
 {
     DisplayInfo *display;
+#ifdef HAVE_XSYNC
+    int xsync_major, xsync_minor;
+#endif /* HAVE_XSYNC */
     int dummy;
 
     display = g_new0 (DisplayInfo, 1);
@@ -164,6 +191,10 @@ myDisplayInit (GdkDisplay *gdisplay)
     display->gdisplay = gdisplay;
     display->dpy = (Display *) gdk_x11_display_get_xdisplay (gdisplay);
 
+    display->session = NULL;
+    display->quit = FALSE;
+    display->reload = FALSE;
+
     XSetErrorHandler (handleXError);
 
     /* Initialize internal atoms */
@@ -173,18 +204,87 @@ myDisplayInit (GdkDisplay *gdisplay)
     }
 
     /* Test XShape extension support */
-    display->shape = 
-        XShapeQueryExtension (display->dpy, &display->shape_event, &dummy);
-    if (!display->shape)
+    if (XShapeQueryExtension (display->dpy,
+                              &display->shape_event_base,
+                              &dummy))
+    {
+        display->have_shape = TRUE;
+    }
+    else
     {
         g_warning ("The display does not support the XShape extension.");
+        display->have_shape = FALSE;
+        display->shape_event_base = 0;
     }
 
-    display->root_cursor = 
-        XCreateFontCursor (display->dpy, XC_left_ptr);
-    display->move_cursor = 
-        XCreateFontCursor (display->dpy, XC_fleur);
-    display->busy_cursor = 
+#ifdef HAVE_XSYNC
+    display->have_xsync = FALSE;
+    
+    display->xsync_error_base = 0;
+    display->xsync_event_base = 0;
+
+    xsync_major = SYNC_MAJOR_VERSION;
+    xsync_minor = SYNC_MINOR_VERSION;
+    
+    if (XSyncQueryExtension (display->dpy,
+                              &display->xsync_event_base,
+                              &display->xsync_error_base)
+         && XSyncInitialize (display->dpy, 
+                             &xsync_major, 
+                             &xsync_minor))
+    {
+        display->have_xsync = TRUE;
+    }
+    else
+    {
+        g_warning ("The display does not support the XSync extension.");
+        display->have_xsync = FALSE;
+        display->xsync_event_base = 0;
+        display->xsync_error_base = 0;
+    }
+#endif /* HAVE_XSYNC */
+
+#ifdef HAVE_RENDER
+    if (XRenderQueryExtension (display->dpy,
+                               &display->render_event_base,
+                               &display->render_error_base))
+    {
+        display->have_render = TRUE;
+    }
+    else
+    {
+        g_warning ("The display does not support the XRender extension.");
+        display->have_render = FALSE;
+        display->render_event_base = 0;
+        display->render_error_base = 0;
+    }
+#else  /* HAVE_RENDER */
+    display->have_render = FALSE;
+#endif /* HAVE_RENDER */
+
+#ifdef HAVE_RANDR
+    if (XRRQueryExtension (display->dpy,
+                            &display->xrandr_event_base,
+                            &display->xrandr_error_base))
+    {
+        display->have_xrandr = TRUE;
+    }
+    else
+    {
+        g_warning ("The display does not support the XRandr extension.");
+        display->have_xrandr = FALSE;
+        display->xrandr_event_base = 0;
+        display->xrandr_error_base = 0;
+    }
+#else  /* HAVE_RANDR */
+    display->have_xrandr = FALSE;
+#endif /* HAVE_RANDR */
+
+    display->root_cursor =
+        XCreateFontCursor (display->dpy, CURSOR_ROOT);
+    display->move_cursor =
+        XCreateFontCursor (display->dpy, CURSOR_MOVE);
+    display->busy_cursor =
         cursorCreateSpinning (display->dpy);
     display->resize_cursor[CORNER_TOP_LEFT] =
         XCreateFontCursor (display->dpy, XC_top_left_corner);
@@ -194,11 +294,13 @@ myDisplayInit (GdkDisplay *gdisplay)
         XCreateFontCursor (display->dpy, XC_bottom_left_corner);
     display->resize_cursor[CORNER_BOTTOM_RIGHT] =
         XCreateFontCursor (display->dpy, XC_bottom_right_corner);
-    display->resize_cursor[4 + SIDE_LEFT] = 
+    display->resize_cursor[CORNER_COUNT + SIDE_LEFT] =
         XCreateFontCursor (display->dpy, XC_left_side);
-    display->resize_cursor[4 + SIDE_RIGHT] = 
+    display->resize_cursor[CORNER_COUNT + SIDE_RIGHT] =
         XCreateFontCursor (display->dpy, XC_right_side);
-    display->resize_cursor[4 + SIDE_BOTTOM] = 
+    display->resize_cursor[CORNER_COUNT + SIDE_TOP] =
+        XCreateFontCursor (display->dpy, XC_top_side);
+    display->resize_cursor[CORNER_COUNT + SIDE_BOTTOM] =
         XCreateFontCursor (display->dpy, XC_bottom_side);
 
     display->xfilter = NULL;
@@ -208,7 +310,14 @@ myDisplayInit (GdkDisplay *gdisplay)
     display->dbl_click_time = 300;
     display->nb_screens = 0;
     display->current_time = CurrentTime;
-    
+
+    display->hostname = g_new0 (gchar, (size_t) MAX_HOSTNAME_LENGTH);
+    if (gethostname ((char *) display->hostname, MAX_HOSTNAME_LENGTH - 1))
+    {
+        g_free (display->hostname);
+        display->hostname = NULL;
+    }
+
     compositorInitDisplay (display);
 
     return display;
@@ -225,51 +334,73 @@ myDisplayClose (DisplayInfo *display)
     display->move_cursor = None;
     XFreeCursor (display->dpy, display->root_cursor);
     display->root_cursor = None;
-    
+
+    if (display->hostname)
+    {
+        g_free (display->hostname);
+        display->hostname = NULL;
+    }
+
     for (i = 0; i < 7; i++)
     {
         XFreeCursor (display->dpy, display->resize_cursor[i]);
         display->resize_cursor[i] = None;
     }
-    
+
     g_slist_free (display->clients);
     display->clients = NULL;
-    
+
     g_slist_free (display->screens);
     display->screens = NULL;
 
     return display;
 }
 
-Cursor 
+gboolean
+myDisplayHaveShape (DisplayInfo *display)
+{
+    g_return_val_if_fail (display != NULL, FALSE);
+
+    return (display->have_shape);
+}
+
+gboolean
+myDisplayHaveRender (DisplayInfo *display)
+{
+    g_return_val_if_fail (display != NULL, FALSE);
+
+    return (display->have_render);
+}
+
+Cursor
 myDisplayGetCursorBusy (DisplayInfo *display)
 {
     g_return_val_if_fail (display, None);
-    
+
     return display->busy_cursor;
 }
 
-Cursor 
+Cursor
 myDisplayGetCursorMove  (DisplayInfo *display)
 {
     g_return_val_if_fail (display, None);
-    
+
     return display->move_cursor;
 }
 
-Cursor 
+Cursor
 myDisplayGetCursorRoot (DisplayInfo *display)
 {
     g_return_val_if_fail (display, None);
-    
+
     return display->root_cursor;
 }
 
-Cursor 
+Cursor
 myDisplayGetCursorResize (DisplayInfo *display, guint index)
 {
     g_return_val_if_fail (display, None);
-    g_return_val_if_fail (index < 7, None);
+    g_return_val_if_fail (index < 8, None);
 
     return display->resize_cursor [index];
 }
@@ -279,7 +410,7 @@ void
 myDisplayGrabServer (DisplayInfo *display)
 {
     g_return_if_fail (display);
-    
+
     DBG ("entering myDisplayGrabServer");
     if (display->xgrabcount == 0)
     {
@@ -308,7 +439,7 @@ myDisplayUngrabServer (DisplayInfo *display)
     DBG ("grabs : %i", display->xgrabcount);
 }
 
-void 
+void
 myDisplayAddClient (DisplayInfo *display, Client *c)
 {
     g_return_if_fail (c != None);
@@ -317,7 +448,7 @@ myDisplayAddClient (DisplayInfo *display, Client *c)
     display->clients = g_slist_append (display->clients, c);
 }
 
-void 
+void
 myDisplayRemoveClient (DisplayInfo *display, Client *c)
 {
     g_return_if_fail (c != None);
@@ -368,7 +499,7 @@ myDisplayGetClientFromWindow (DisplayInfo *display, Window w, int mode)
     return NULL;
 }
 
-void 
+void
 myDisplayAddScreen (DisplayInfo *display, ScreenInfo *screen)
 {
     g_return_if_fail (screen != NULL);
@@ -378,7 +509,7 @@ myDisplayAddScreen (DisplayInfo *display, ScreenInfo *screen)
     display->nb_screens = display->nb_screens + 1;
 }
 
-void 
+void
 myDisplayRemoveScreen (DisplayInfo *display, ScreenInfo *screen)
 {
     g_return_if_fail (screen != NULL);
@@ -437,18 +568,13 @@ Window
 myDisplayGetRootFromWindow(DisplayInfo *display, Window w)
 {
     XWindowAttributes attributes;
-    gint test;
 
     g_return_val_if_fail (w != None, None);
     g_return_val_if_fail (display != NULL, None);
 
-    gdk_error_trap_push ();
-    XGetWindowAttributes(display->dpy, w, &attributes);
-    test = gdk_error_trap_pop ();
-
-    if (test)
+    if (!XGetWindowAttributes(display->dpy, w, &attributes))
     {
-        TRACE ("myDisplayGetRootFromWindow: no root found for 0x%lx, error code %i", w, test);
+        TRACE ("myDisplayGetRootFromWindow: no root found for 0x%lx", w);
         return None;
     }
     return attributes.root;
@@ -472,6 +598,7 @@ myDisplayGetScreenFromWindow (DisplayInfo *display, Window w)
     return NULL;
 }
 
+#ifdef ENABLE_KDE_SYSTRAY_PROXY
 ScreenInfo *
 myDisplayGetScreenFromSystray (DisplayInfo *display, Window w)
 {
@@ -492,12 +619,52 @@ myDisplayGetScreenFromSystray (DisplayInfo *display, Window w)
 
     return NULL;
 }
+#endif /* ENABLE_KDE_SYSTRAY_PROXY */
 
-Time 
-myDisplayUpdateCurentTime (DisplayInfo *display, XEvent *ev)
+#ifdef HAVE_XSYNC
+Client *
+myDisplayGetClientFromXSyncAlarm (DisplayInfo *display, XSyncAlarm alarm)
+{
+    GSList *index;
+
+    g_return_val_if_fail (alarm != None, NULL);
+    g_return_val_if_fail (display != NULL, NULL);
+    
+    for (index = display->clients; index; index = g_slist_next (index))
+    {
+        Client *c = (Client *) index->data;
+        if (alarm == c->xsync_alarm)
+        {
+            return (c);
+        }
+    }
+    TRACE ("no client found");
+
+    return NULL;
+}
+#endif /* HAVE_XSYNC */
+
+ScreenInfo *
+myDisplayGetDefaultScreen (DisplayInfo *display)
+{
+    GSList *index;
+
+    g_return_val_if_fail (display != NULL, NULL);
+
+    index = display->screens;
+    if (index)
+    {
+        return (ScreenInfo *) index->data;
+    }
+
+    return NULL;
+}
+
+Time
+myDisplayUpdateCurrentTime (DisplayInfo *display, XEvent *ev)
 {
     g_return_val_if_fail (display != NULL, (Time) CurrentTime);
-    
+
     switch (ev->type)
     {
         case KeyPress:
@@ -528,15 +695,166 @@ myDisplayUpdateCurentTime (DisplayInfo *display, XEvent *ev)
             display->current_time = (Time) ev->xselection.time;
             break;
         default:
+            display->current_time = (Time) CurrentTime;
             break;
     }
     return display->current_time;
 }
 
-Time 
+Time
 myDisplayGetCurrentTime (DisplayInfo *display)
 {
     g_return_val_if_fail (display != NULL, (Time) CurrentTime);
 
     return (Time) display->current_time;
 }
+
+Time
+myDisplayGetLastUserTime (DisplayInfo *display)
+{
+    g_return_val_if_fail (display != NULL, (Time) CurrentTime);
+
+    return (Time) display->last_user_time;
+}
+
+void
+myDisplaySetLastUserTime (DisplayInfo *display, Time timestamp)
+{
+    g_return_if_fail (display != NULL);
+
+    if (TIMESTAMP_IS_BEFORE(display->last_user_time, timestamp))
+    {
+        display->last_user_time = timestamp;
+    }
+}
+
+gboolean
+myDisplayTestXrender (DisplayInfo *display, gdouble min_time)
+{
+#ifdef HAVE_RENDER
+    GTimeVal t1, t2;
+    gdouble dt;
+    Display *dpy;
+    Picture picture1, picture2, picture3;
+    XRenderPictFormat *format_src, *format_dst;
+    Pixmap fillPixmap, rootPixmap;
+    XRenderPictureAttributes pa;
+    XSetWindowAttributes attrs;
+    XImage *ximage;
+    Window output;
+    XRenderColor c;
+    Visual *visual;
+    Screen *screen;
+    int x, y, w, h;
+    int screen_number;
+    int depth;
+    int iterations;
+
+    g_return_val_if_fail (display != NULL, FALSE);
+    TRACE ("entering myDisplayTesxrender");
+
+    c.alpha = 0x7FFF;
+    c.red   = 0xFFFF;
+    c.green = 0xFFFF;
+    c.blue  = 0xFFFF;
+
+    dpy = display->dpy;
+    screen_number = DefaultScreen (dpy);
+    screen = DefaultScreenOfDisplay (dpy);
+    visual = DefaultVisual (dpy, screen_number);
+    depth = DefaultDepth (dpy, screen_number);
+
+    w = WidthOfScreen(screen) / 16;
+    h = HeightOfScreen(screen) / 16;
+    x = (WidthOfScreen(screen) - w);
+    y = (HeightOfScreen(screen) - h);
+
+    format_dst = XRenderFindVisualFormat (dpy, visual);
+    g_return_val_if_fail (format_dst != NULL , FALSE);
+
+    format_src = XRenderFindStandardFormat (dpy, PictStandardA8);
+    g_return_val_if_fail (format_src != NULL , FALSE);
+
+    ximage = XGetImage (dpy,
+                        DefaultRootWindow(dpy),
+                        x, y, w, h,
+                        AllPlanes, ZPixmap);
+    g_return_val_if_fail (ximage != NULL , FALSE);
+
+    rootPixmap = XCreatePixmap (dpy,
+                                DefaultRootWindow(dpy),
+                                w, h, depth);
+    XPutImage (dpy, rootPixmap,
+               DefaultGC (dpy, screen_number), ximage,
+               0, 0, 0, 0, w, h);
+    XDestroyImage (ximage);
+
+    attrs.override_redirect = TRUE;
+    output = XCreateWindow (dpy,
+                            DefaultRootWindow(dpy),
+                            x, y, w, h,
+                            0, CopyFromParent, CopyFromParent,
+                            (Visual *) CopyFromParent,
+                            CWOverrideRedirect, &attrs);
+    XMapRaised (dpy, output);
+
+    fillPixmap = XCreatePixmap (dpy,
+                                DefaultRootWindow(dpy),
+                                1, 1, 8);
+
+    g_get_current_time (&t1);
+
+    pa.repeat = TRUE;
+    picture1 = XRenderCreatePicture (dpy,
+                                     rootPixmap,
+                                     format_dst, 0, NULL);
+    picture2 = XRenderCreatePicture (dpy,
+                                     fillPixmap,
+                                     format_src, CPRepeat, &pa);
+    picture3 = XRenderCreatePicture (dpy,
+                                     output,
+                                     format_dst, 0, NULL);
+    XRenderComposite (dpy, PictOpSrc,
+                    picture1, None, picture3,
+                    0, 0, 0, 0, 0, 0, w, h);
+    XRenderFillRectangle (dpy, PictOpSrc,
+                          picture2, &c, 0, 0,
+                          1, 1);
+    for (iterations = 0; iterations < 10; iterations++)
+    {
+        XRenderComposite (dpy, PictOpOver,
+                        picture1, picture2, picture3,
+                        0, 0, 0, 0, 0, 0, w, h);
+        ximage = XGetImage (dpy, output,
+                            0, 0, 1, 1,
+                            AllPlanes, ZPixmap);
+        if (ximage)
+        {
+                XDestroyImage (ximage);
+        }
+    }
+    XRenderFreePicture (dpy, picture1);
+    XRenderFreePicture (dpy, picture2);
+    XRenderFreePicture (dpy, picture3);
+
+    XFreePixmap (dpy, fillPixmap);
+    XFreePixmap (dpy, rootPixmap);
+
+    XDestroyWindow (dpy, output);
+
+    g_get_current_time (&t2);
+
+    dt = (gdouble) (t2.tv_sec - t1.tv_sec) * G_USEC_PER_SEC +
+         (gdouble) (t2.tv_usec - t1.tv_usec) / 1000.0;
+
+    if (dt < min_time)
+    {
+        TRACE ("XRender test passed (target %3.4f sec., measured %3.4f sec.).", min_time, dt);
+        return TRUE;
+    }
+    g_print ("XRender test failed (target %3.4f sec., measured %3.4f sec.).\n", min_time, dt);
+    return FALSE;
+#else  /* HAVE_RENDER */
+    return FALSE;
+#endif /* HAVE_RENDER */
+}
