@@ -1,21 +1,22 @@
-/*
+/*      $Id: events.h 23704 2006-11-04 22:36:42Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2003 Olivier Fourdan
- 
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
  */
 
 #ifndef INC_EVENTS_H
@@ -31,11 +32,13 @@
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <glib.h>
-#include <libxfcegui4/libxfcegui4.h>
 
+#include "screen.h"
+#include "event_filter.h"
 
-void handleEvent (XEvent *);
-GtkToXEventFilterStatus xfwm4_event_filter (XEvent * xevent, gpointer data);
-void initGtkCallbacks (void);
+void                     initMenuEventWin                       (void);
+eventFilterStatus        xfwm4_event_filter                     (XEvent *, 
+                                                                 gpointer);
+void                     initGtkCallbacks                       (ScreenInfo *);
 
 #endif /* INC_EVENTS_H */
