@@ -1,24 +1,24 @@
-/*      $Id: focus.c 22892 2006-08-26 21:02:28Z olivier $
- 
+/*      $Id: focus.c 23551 2006-10-29 22:59:52Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
- 
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
  */
- 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -32,8 +32,7 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <libxfce4util/libxfce4util.h> 
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4util/libxfce4util.h>
 
 #include "screen.h"
 #include "focus.h"
@@ -61,8 +60,8 @@ static ClientPair
 clientGetTopMostFocusable (ScreenInfo *screen_info, int layer, Client * exclude)
 {
     ClientPair top_client;
-    Client *c = NULL;
-    GList *index = NULL;
+    Client *c;
+    GList *index;
 
     TRACE ("entering clientGetTopMostFocusable");
 
@@ -73,12 +72,11 @@ clientGetTopMostFocusable (ScreenInfo *screen_info, int layer, Client * exclude)
         TRACE ("*** stack window \"%s\" (0x%lx), layer %i", c->name,
             c->window, (int) c->win_layer);
 
-        if ((c->type & (WINDOW_SPLASHSCREEN | WINDOW_DOCK | WINDOW_DESKTOP))
-            || ((layer != WIN_LAYER_DESKTOP) && (c->type & WINDOW_DESKTOP)))
+        if (!clientAcceptFocus (c) || (c->type & WINDOW_TYPE_DONT_FOCUS))
         {
             continue;
         }
-        
+
         if (!exclude || (c != exclude))
         {
             if ((c->win_layer <= layer) && FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE))
@@ -103,88 +101,109 @@ void
 clientFocusTop (ScreenInfo *screen_info, int layer)
 {
     ClientPair top_client;
-    DisplayInfo *display_info = NULL;
+    DisplayInfo *display_info;
 
     display_info = screen_info->display_info;
     top_client = clientGetTopMostFocusable (screen_info, layer, NULL);
     if (top_client.prefered)
     {
-        clientSetFocus (screen_info, top_client.prefered, 
-                        myDisplayGetCurrentTime (screen_info->display_info), 
+        clientSetFocus (screen_info, top_client.prefered,
+                        myDisplayGetCurrentTime (screen_info->display_info),
                         NO_FOCUS_FLAG);
     }
     else
     {
-        clientSetFocus (screen_info, top_client.highest, 
-                        myDisplayGetCurrentTime (screen_info->display_info), 
+        clientSetFocus (screen_info, top_client.highest,
+                        myDisplayGetCurrentTime (screen_info->display_info),
                         NO_FOCUS_FLAG);
     }
 }
 
-void
+gboolean
 clientFocusNew(Client * c)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
     gboolean give_focus;
     gboolean prevent_focus_stealing;
+    gboolean prevented;
+
+    g_return_val_if_fail (c != NULL, FALSE);
 
-    g_return_if_fail (c != NULL);
-    
-    if (!clientAcceptFocus (c)|| (c->type & WINDOW_TYPE_DONT_FOCUS))
-    {
-        return;
-    }
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
     give_focus = (c-> type & WINDOW_REGULAR_FOCUSABLE) && (screen_info->params->focus_new);
     prevent_focus_stealing = screen_info->params->prevent_focus_stealing;
+    prevented = FALSE;
 
     /*  Try to avoid focus stealing */
-    if ((client_focus) && (prevent_focus_stealing))
+    if (!clientAcceptFocus (c) || (c->type & WINDOW_TYPE_DONT_FOCUS))
     {
-        if (FLAG_TEST(c->flags, CLIENT_FLAG_HAS_USER_TIME) &&
-            FLAG_TEST(client_focus->flags, CLIENT_FLAG_HAS_USER_TIME))
+        give_focus = FALSE;
+    }
+    else if ((client_focus) && (prevent_focus_stealing))
+    {
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_STARTUP_TIME) && (c->user_time == (Time) 0))
+        {
+            TRACE ("Given startup time is 0, not focusing");
+            give_focus = FALSE;
+            prevented = TRUE;
+        }
+        else if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_STARTUP_TIME | CLIENT_FLAG_HAS_USER_TIME))
         {
-            TRACE ("Current %u, new %u", (unsigned int) client_focus->user_time,(unsigned int)  c->user_time);
             if (c->user_time < client_focus->user_time)
             {
+                TRACE ("Current %u, new %u", (unsigned int) client_focus->user_time, (unsigned int) c->user_time);
                 give_focus = FALSE;
+                prevented = TRUE;
             }
         }
     }
 
-    clientGrabMouseButton (c);
-    if (give_focus || FLAG_TEST(c->flags, CLIENT_FLAG_STATE_MODAL))
+    if ((give_focus) || FLAG_TEST(c->flags, CLIENT_FLAG_STATE_MODAL))
     {
+        give_focus = TRUE;
         if ((client_focus) && !(clientIsTransientOrModalFor (c, client_focus)))
         {
-            if (clientAdjustFullscreenLayer (client_focus, FALSE))
-            {
-                clientRaise (c);
-            }
+            clientAdjustFullscreenLayer (client_focus, FALSE);
         }
-        clientSetFocus (screen_info, c, 
-#if 0
-                        myDisplayGetCurrentTime (display_info), 
-#else
-                        CurrentTime, 
-#endif
+        clientRaise (c, None);
+        clientShow (c, TRUE);
+        clientSetFocus (screen_info, c,
+                        myDisplayGetCurrentTime (display_info),
                         FOCUS_IGNORE_MODAL);
     }
     else
     {
-        FLAG_SET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
+        Client *c2 = clientGetFocus();
+
+        if (prevented && c2 && (c2->win_layer == c->win_layer))
+        {
+            FLAG_SET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
+            clientSortRing(c);
+            clientLower (c, c2->frame);
+            clientSortRing(c2);
+            clientSetOpacity (c, c->opacity, 0, 0);
+        }
+        else
+        {
+            clientRaise (c, None);
+            clientSortRing(c);
+        }
+
+        clientShow (c, TRUE);
         clientSetNetState (c);
+        clientGrabMouseButton (c);
     }
+
+    return (give_focus);
 }
 
 gboolean
 clientSelectMask (Client * c, int mask, int type)
 {
-    TRACE ("entering clientSelectMask");
-
     g_return_val_if_fail (c != NULL, FALSE);
+    TRACE ("entering clientSelectMask");
 
     if ((!clientAcceptFocus (c)) && !(mask & INCLUDE_SKIP_FOCUS))
     {
@@ -219,7 +238,7 @@ clientSelectMask (Client * c, int mask, int type)
 Client *
 clientGetNext (Client * c, int mask)
 {
-    Client *c2 = NULL;
+    Client *c2;
     unsigned int i;
 
     TRACE ("entering clientGetNext");
@@ -242,7 +261,7 @@ clientGetNext (Client * c, int mask)
 Client *
 clientGetPrevious (Client * c, int mask)
 {
-    Client *c2 = NULL;
+    Client *c2;
     unsigned int i;
 
     TRACE ("entering clientGetPrevious");
@@ -265,18 +284,23 @@ clientGetPrevious (Client * c, int mask)
 void
 clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
 {
-    DisplayInfo *display_info = NULL;
-    Client *new_focus = NULL;
-    Client *current_focus = client_focus;
+    DisplayInfo *display_info;
     ClientPair top_most;
-    Client *c2 = NULL;
+    Client *new_focus;
+    Client *current_focus;
+    Client *c2;
     Window dr, window;
-    int rx, ry, wx, wy;
     unsigned int mask;
-    int look_in_layer = (c ? c->win_layer : WIN_LAYER_NORMAL);
+    int rx, ry, wx, wy;
+    int look_in_layer;
 
     TRACE ("entering clientPassFocus");
 
+    look_in_layer = (c ? c->win_layer : WIN_LAYER_NORMAL);
+    new_focus = NULL;
+    current_focus = client_focus;
+    c2 = NULL;
+
     if (pending_focus)
     {
         current_focus = pending_focus;
@@ -300,9 +324,9 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
         {
             if (clientIsModal (c))
             {
-                /* If the window is a modal, send focus back to its parent window
-                   Modals are transients, and we aren"t interested in modal
-                   for group, so it safe to use clientGetTransient because 
+                /* If the window is a modal, send focus back to its parent window.
+                   Modals are transients, and we aren't interested in modal
+                   for group, so it safe to use clientGetTransient because
                    it's really what we want...
                  */
                 c2 = clientGetTransient (c);
@@ -310,9 +334,10 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
                 {
                     new_focus = c2;
                     /* Usability: raise the parent, to grab user's attention */
-                    clientRaise (c2);
+                    clientRaise (c2, None);
                 }
             }
+#if 0
             else
             {
                 c2 = clientGetNext (c, 0);
@@ -321,6 +346,7 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
                     new_focus = c2;
                 }
             }
+#endif
         }
     }
     else if (XQueryPointer (myScreenGetXDisplay (screen_info), screen_info->xroot, &dr, &window, &rx, &ry, &wx, &wy, &mask))
@@ -331,12 +357,8 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
     {
         new_focus = top_most.prefered ? top_most.prefered : top_most.highest;
     }
-    clientSetFocus (screen_info, new_focus, 
-#if 0
-                    myDisplayGetCurrentTime (screen_info->display_info), 
-#else
-                    CurrentTime, 
-#endif
+    clientSetFocus (screen_info, new_focus,
+                    myDisplayGetCurrentTime (screen_info->display_info),
                     FOCUS_IGNORE_MODAL | FOCUS_FORCE);
 }
 
@@ -344,37 +366,37 @@ gboolean
 clientAcceptFocus (Client * c)
 {
     g_return_val_if_fail (c != NULL, FALSE);
-
     TRACE ("entering clientAcceptFocus");
 
     /* Modal dialogs *always* accept focus */
     if (FLAG_TEST(c->flags, CLIENT_FLAG_STATE_MODAL))
     {
-        return TRUE; 
+        return TRUE;
     }
     /* First check GNOME protocol */
     if (c->win_hints & WIN_HINTS_SKIP_FOCUS)
     {
         return FALSE;
     }
-    if (!FLAG_TEST (c->wm_flags, WM_FLAG_INPUT | WM_FLAG_TAKEFOCUS))
+    if ((c->screen_info->params->focus_hint)
+        && !FLAG_TEST (c->wm_flags, WM_FLAG_INPUT | WM_FLAG_TAKEFOCUS))
     {
         return FALSE;
     }
-    
+
     return TRUE;
 }
 
 void
 clientSortRing(Client *c)
 {
-    ScreenInfo *screen_info = NULL;
-    
-    g_return_if_fail (c != NULL);
+    ScreenInfo *screen_info;
 
+    g_return_if_fail (c != NULL);
     TRACE ("Sorting...");
+
     screen_info = c->screen_info;
-    if (screen_info->client_count > 2 && c != screen_info->clients)
+    if ((screen_info->client_count > 2) && (c != screen_info->clients))
     {
         c->prev->next = c->next;
         c->next->prev = c->prev;
@@ -390,12 +412,13 @@ clientSortRing(Client *c)
 void
 clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
 {
-    DisplayInfo *display_info = NULL;
-    Client *c2 = ((client_focus != c) ? client_focus : NULL);
+    DisplayInfo *display_info;
+    Client *c2;
     unsigned long data[2];
 
     TRACE ("entering clientUpdateFocus");
 
+    c2 = ((client_focus != c) ? client_focus : NULL);
     display_info = screen_info->display_info;
     pending_focus = NULL;
     if ((c) && !clientAcceptFocus (c))
@@ -410,7 +433,7 @@ clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
         return;
     }
 
-    /* 
+    /*
        We can release the button mouse grab if we don't raise on click or if the focused window
        is the one that has been raised at last.
      */
@@ -429,7 +452,7 @@ clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
         }
         data[0] = c->window;
         clientAdjustFullscreenLayer (c, TRUE);
-        frameDraw (c, FALSE, FALSE);
+        frameDraw (c, FALSE);
     }
     else
     {
@@ -440,25 +463,28 @@ clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
         if (c)
         {
             clientAdjustFullscreenLayer (c2, FALSE);
-            /* clientRaise (c); */
+            /* clientRaise (c, None); */
         }
-        frameDraw (c2, FALSE, FALSE);
+        frameDraw (c2, FALSE);
     }
     data[1] = None;
-    XChangeProperty (display_info->dpy, screen_info->xroot, 
+    XChangeProperty (display_info->dpy, screen_info->xroot,
                      display_info->atoms[NET_ACTIVE_WINDOW], XA_WINDOW, 32,
                      PropModeReplace, (unsigned char *) data, 2);
+
+    clientUpdateOpacity (screen_info, c);
 }
 
 void
 clientSetFocus (ScreenInfo *screen_info, Client * c, Time timestamp, unsigned short flags)
 {
-    DisplayInfo *display_info = NULL;
-    Client *c2 = NULL;
+    DisplayInfo *display_info;
+    Client *c2;
 
     TRACE ("entering clientSetFocus");
-    
+
     display_info = screen_info->display_info;
+    c2 = NULL;
     if ((c) && !(flags & FOCUS_IGNORE_MODAL))
     {
         c2 = clientGetModalFor (c);
@@ -472,52 +498,94 @@ clientSetFocus (ScreenInfo *screen_info, Client * c, Time timestamp, unsigned sh
     if ((c) && FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE))
     {
         TRACE ("setting focus to client \"%s\" (0x%lx)", c->name, c->window);
-        if ((c == client_focus) && !(flags & FOCUS_FORCE))
-        {
-            TRACE ("client \"%s\" (0x%lx) is already focused, ignoring request",
-                c->name, c->window);
-            return;
-        }        
-        if (!clientAcceptFocus (c))
-        {
-            TRACE ("SKIP_FOCUS set for client \"%s\" (0x%lx)", c->name, c->window);
-            return;
-        }
-        if (FLAG_TEST (c->wm_flags, WM_FLAG_INPUT))
-        {
-            pending_focus = c;
-            XSetInputFocus (myScreenGetXDisplay (screen_info), c->window, RevertToPointerRoot, timestamp);
-        }
-        if (FLAG_TEST(c->wm_flags, WM_FLAG_TAKEFOCUS))
-        {
-            sendClientMessage (c->screen_info, c->window, WM_TAKE_FOCUS, timestamp);
-        }
         if (FLAG_TEST(c->flags, CLIENT_FLAG_DEMANDS_ATTENTION))
         {
             FLAG_UNSET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
             clientSetNetState (c);
         }
+        if ((c == client_focus) && !(flags & FOCUS_FORCE))
+        {
+            TRACE ("client \"%s\" (0x%lx) is already focused, ignoring request",
+                c->name, c->window);
+            return;
+        }
+        if (!clientAcceptFocus (c))
+        {
+            TRACE ("SKIP_FOCUS set for client \"%s\" (0x%lx)", c->name, c->window);
+            return;
+        }
+        if (FLAG_TEST (c->wm_flags, WM_FLAG_INPUT) || !(screen_info->params->focus_hint))
+        {
+            pending_focus = c;
+            /*
+             * When shaded, the client window is unmapped, so it can not be focused.
+             * Instead, we focus the frame that is still mapped.
+             */
+            if (FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
+            {
+                XSetInputFocus (myScreenGetXDisplay (screen_info), c->frame, RevertToPointerRoot, timestamp);
+            }
+            else
+            {
+                XSetInputFocus (myScreenGetXDisplay (screen_info), c->window, RevertToPointerRoot, timestamp);
+            }
+            clientUpdateOpacity (screen_info, c);
+        }
+        if (FLAG_TEST(c->wm_flags, WM_FLAG_TAKEFOCUS))
+        {
+            sendClientMessage (c->screen_info, c->window, WM_TAKE_FOCUS, timestamp);
+        }
     }
     else
     {
         unsigned long data[2];
-        
+
         TRACE ("setting focus to none");
-        
+
         data[0] = data[1] = None;
         client_focus = NULL;
         if (c2)
         {
-            frameDraw (c2, FALSE, FALSE);
+            frameDraw (c2, FALSE);
             XChangeProperty (clientGetXDisplay (c2), c2->screen_info->xroot, display_info->atoms[NET_ACTIVE_WINDOW], XA_WINDOW, 32,
                              PropModeReplace, (unsigned char *) data, 2);
         }
         XChangeProperty (myScreenGetXDisplay (screen_info), screen_info->xroot, display_info->atoms[NET_ACTIVE_WINDOW], XA_WINDOW, 32,
                          PropModeReplace, (unsigned char *) data, 2);
-        XSetInputFocus (myScreenGetXDisplay (screen_info), screen_info->gnome_win, RevertToPointerRoot, timestamp);
+        XSetInputFocus (myScreenGetXDisplay (screen_info), screen_info->xfwm4_win, RevertToPointerRoot, timestamp);
+        clientUpdateOpacity (screen_info, c);
     }
 }
 
+void
+clientInitFocusFlag (Client * c)
+{
+    ScreenInfo *screen_info;
+    Client *c2;
+    GList *index;
+    int workspace;
+
+    g_return_if_fail (c != NULL);
+    TRACE ("entering clientSetFocus");
+
+    if (!clientAcceptFocus (c) || (c->type & WINDOW_TYPE_DONT_FOCUS))
+    {
+       return;
+    }
+
+    screen_info = c->screen_info;
+    workspace = c->win_workspace;
+    for (index = screen_info->windows_stack; index; index = g_list_next (index))
+    {
+        c2 = (Client *) index->data;
+        if ((c2->win_workspace == workspace) && FLAG_TEST (c2->xfwm_flags, XFWM_FLAG_FOCUS))
+        {
+            FLAG_UNSET (c2->xfwm_flags, XFWM_FLAG_FOCUS);
+        }
+    }
+    FLAG_SET (c->xfwm_flags, XFWM_FLAG_FOCUS);
+}
+
 Client *
 clientGetFocus (void)
 {
@@ -533,8 +601,8 @@ clientClearFocus (void)
 void
 clientGrabMouseButton (Client * c)
 {
-    ScreenInfo *screen_info = NULL;
-    
+    ScreenInfo *screen_info;
+
     g_return_if_fail (c != NULL);
     TRACE ("entering clientGrabMouseButton");
     TRACE ("grabbing buttons for client \"%s\" (0x%lx)", c->name, c->window);
@@ -542,19 +610,19 @@ clientGrabMouseButton (Client * c)
     screen_info = c->screen_info;
     if (screen_info->params->raise_with_any_button)
     {
-        grabButton(clientGetXDisplay (c), AnyButton, 0, c->window);
+        grabButton(clientGetXDisplay (c), AnyButton, AnyModifier, c->window);
     }
     else
     {
-        grabButton(clientGetXDisplay (c), Button1, 0, c->window);
+        grabButton(clientGetXDisplay (c), Button1, AnyModifier, c->window);
     }
 }
 
 void
 clientUngrabMouseButton (Client * c)
 {
-    ScreenInfo *screen_info = NULL;
-    
+    ScreenInfo *screen_info;
+
     g_return_if_fail (c != NULL);
     TRACE ("entering clientUngrabMouseButton");
     TRACE ("ungrabing buttons for client \"%s\" (0x%lx)", c->name, c->window);
@@ -570,7 +638,7 @@ clientGrabMouseButtonForAll (ScreenInfo *screen_info)
 {
     Client *c;
     int i;
-    
+
     g_return_if_fail (screen_info != NULL);
     TRACE ("entering clientGrabMouseButtonForAll");
 
@@ -586,7 +654,7 @@ clientUngrabMouseButtonForAll (ScreenInfo *screen_info)
 {
     Client *c;
     int i;
-    
+
     g_return_if_fail (screen_info != NULL);
     TRACE ("entering clientUngrabMouseButtonForAll");
 
