@@ -1,23 +1,23 @@
-/*      $Id: hints.c 21734 2006-05-20 19:54:52Z olivier $
- 
+/*      $Id: hints.c 23704 2006-11-04 22:36:42Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+
         oroborus - (c) 2001 Ken Lynch
         Metacity - (c) 2001 Havoc Pennington
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
- 
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,8 +35,9 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <libxfce4util/libxfce4util.h> 
+#include <libxfce4util/libxfce4util.h>
 #include "display.h"
+#include "screen.h"
 #include "hints.h"
 
 static gboolean
@@ -49,17 +50,58 @@ check_type_and_format (int expected_format, Atom expected_type, int n_items, int
     return FALSE;
 }
 
+static gchar *
+internal_utf8_strndup (const gchar *src, gssize max_len)
+{
+    const gchar *s;
+
+    if (max_len <= 0)
+    {
+        return g_strdup (src);
+    }
+
+    s = src;
+    while (max_len && *s)
+    {
+        s = g_utf8_next_char (s);
+        max_len--;
+    }
+
+    return g_strndup (src, s - src);
+}
+
+static gchar*
+create_name_with_host (DisplayInfo *display_info, const gchar *name, const gchar *hostname)
+{
+    gchar *title;
+
+    if (strlen (hostname) && (display_info->hostname) && (g_strcasecmp (display_info->hostname, hostname)))
+    {
+        /* TRANSLATORS: "(on %s)" is like "running on" the name of the other host */
+        title = g_strdup_printf (_("%s (on %s)"), name, hostname);
+    }
+    else
+    {
+        title = g_strdup (name);
+    }
+
+    return title;
+}
+
 unsigned long
 getWMState (DisplayInfo *display_info, Window w)
 {
     Atom real_type;
     int real_format;
     unsigned long items_read, items_left;
-    unsigned long *data = NULL, state = WithdrawnState;
+    unsigned char *data;
+    unsigned long state;
 
     TRACE ("entering getWmState");
 
-    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[WM_STATE], 
+    data = NULL;
+    state = WithdrawnState;
+    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[WM_STATE],
                              0, 3L, FALSE, display_info->atoms[WM_STATE],
                              &real_type, &real_format, &items_read, &items_left,
                              (unsigned char **) &data) == Success) && (items_read))
@@ -83,7 +125,7 @@ setWMState (DisplayInfo *display_info, Window w, unsigned long state)
     data[0] = state;
     data[1] = None;
 
-    XChangeProperty (display_info->dpy, w, display_info->atoms[WM_STATE], 
+    XChangeProperty (display_info->dpy, w, display_info->atoms[WM_STATE],
                      display_info->atoms[WM_STATE], 32, PropModeReplace,
                      (unsigned char *) data, 2);
 }
@@ -94,21 +136,23 @@ getMotifHints (DisplayInfo *display_info, Window w)
     Atom real_type;
     int real_format;
     unsigned long items_read, items_left;
-    long *data = NULL;
-    PropMwmHints *result = NULL;
+    unsigned char *data;
+    PropMwmHints *result;
 
     TRACE ("entering getMotifHints");
 
-    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[MOTIF_WM_HINTS], 0L, MWM_HINTS_ELEMENTS, 
+    data = NULL;
+    result = NULL;
+    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[MOTIF_WM_HINTS], 0L, MWM_HINTS_ELEMENTS,
                 FALSE, display_info->atoms[MOTIF_WM_HINTS], &real_type, &real_format, &items_read,
                 &items_left, (unsigned char **) &data) == Success))
     {
         if (items_read >= MWM_HINTS_ELEMENTS)
-        {    
+        {
             result = g_new0(PropMwmHints, 1);
             memcpy (result, data, sizeof (PropMwmHints));
         }
-        if (data)        
+        if (data)
         {
             XFree (data);
         }
@@ -190,22 +234,24 @@ gboolean
 getHint (DisplayInfo *display_info, Window w, int atom_id, long *value)
 {
     Atom real_type;
-    int real_format;
-    gboolean success = FALSE;
     unsigned long items_read, items_left;
-    long *data = NULL;
+    unsigned char *data;
+    int real_format;
+    gboolean success;
 
-    g_return_val_if_fail (((atom_id > 0) && (atom_id < NB_ATOMS)), FALSE);
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
     TRACE ("entering getHint");
 
+    success = FALSE;
     *value = 0;
+    data = NULL;
 
-    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id], 0L, 1L, 
+    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id], 0L, 1L,
                              FALSE, XA_CARDINAL, &real_type, &real_format, &items_read, &items_left,
                              (unsigned char **) &data) == Success) && (items_read))
     {
-        *value = *data;
-        if (data)        
+        *value = *((long *) data);
+        if (data)
         {
             XFree (data);
         }
@@ -217,10 +263,10 @@ getHint (DisplayInfo *display_info, Window w, int atom_id, long *value)
 void
 setHint (DisplayInfo *display_info, Window w, int atom_id, long value)
 {
-    g_return_if_fail ((atom_id > 0) && (atom_id < NB_ATOMS));
+    g_return_if_fail ((atom_id >= 0) && (atom_id < ATOM_COUNT));
     TRACE ("entering setHint");
 
-    XChangeProperty (display_info->dpy, w, display_info->atoms[atom_id], XA_CARDINAL, 
+    XChangeProperty (display_info->dpy, w, display_info->atoms[atom_id], XA_CARDINAL,
                      32, PropModeReplace, (unsigned char *) &value, 1);
 }
 
@@ -228,14 +274,19 @@ void
 getDesktopLayout (DisplayInfo *display_info, Window root, int ws_count, NetWmDesktopLayout * layout)
 {
     Atom real_type;
-    int real_format;
-    gboolean success = FALSE;
     unsigned long items_read, items_left;
     unsigned long orientation, cols, rows, start;
-    unsigned long *data = NULL;
+    unsigned long *ptr;
+    unsigned char *data;
+    int real_format;
+    gboolean success;
 
     TRACE ("entering getDesktopLayout");
 
+    ptr = NULL;
+    data = NULL;
+    success = FALSE;
+
     if ((XGetWindowProperty (display_info->dpy, root, display_info->atoms[NET_DESKTOP_LAYOUT],
                 0L, 4L, FALSE, XA_CARDINAL,
                 &real_type, &real_format, &items_read, &items_left,
@@ -243,21 +294,22 @@ getDesktopLayout (DisplayInfo *display_info, Window root, int ws_count, NetWmDes
     {
         do
         {
-            orientation = data[0];
-            cols = data[1];
-            rows = data[2];
-            start = (items_read >= 4) ? data[3] : NET_WM_TOPLEFT;
+            ptr = (unsigned long *) data;
+            orientation = (unsigned long) *ptr++;
+            cols = (unsigned long) *ptr++;
+            rows = (unsigned long) *ptr++;
+            start = (items_read >= 4) ? (unsigned long) *ptr++ : NET_WM_TOPLEFT;
 
-            if (orientation > NET_WM_ORIENTATION_VERT) 
+            if (orientation > NET_WM_ORIENTATION_VERT)
             {
                 break;
             }
-            
+
             if (start > NET_WM_BOTTOMLEFT)
             {
                 break;
             }
-            
+
             if ((rows == 0) && (cols == 0))
             {
                 break;
@@ -273,7 +325,7 @@ getDesktopLayout (DisplayInfo *display_info, Window root, int ws_count, NetWmDes
                 cols = (ws_count - 1) / rows + 1;
             }
 
-            layout->orientation = orientation;
+            layout->orientation = (unsigned long) orientation;
             layout->cols = cols;
             layout->rows = rows;
             layout->start = start;
@@ -299,19 +351,23 @@ getGnomeDesktopMargins (DisplayInfo *display_info, Window root, int * m)
     Atom real_type;
     int real_format;
     unsigned long items_read, items_left;
-    unsigned long *data = NULL;
+    unsigned long *ptr;
+    unsigned char *data;
 
     TRACE ("entering getGnomeDesktopMargins");
 
+    ptr = NULL;
+    data = NULL;
     if ((XGetWindowProperty (display_info->dpy, root,
                 display_info->atoms[GNOME_PANEL_DESKTOP_AREA], 0L, 4L, FALSE, XA_CARDINAL,
                 &real_type, &real_format, &items_read, &items_left,
                 (unsigned char **) &data) == Success) && (items_read >= 4))
     {
-        m[0] = (int) data[0];
-        m[1] = (int) data[1];
-        m[2] = (int) data[2];
-        m[3] = (int) data[3];
+        ptr = (unsigned long *) data;
+        m[0] = (int) *ptr++;
+        m[1] = (int) *ptr++;
+        m[2] = (int) *ptr++;
+        m[3] = (int) *ptr++;
         XFree (data);
     }
     else
@@ -340,8 +396,9 @@ setNetSupportedHint (DisplayInfo *display_info, Window root, Window check_win)
 {
     Atom atoms[64];
     unsigned long data[1];
-    int i = 0;
+    int i;
 
+    i = 0;
     atoms[i++] = display_info->atoms[NET_ACTIVE_WINDOW];
     atoms[i++] = display_info->atoms[NET_CLIENT_LIST];
     atoms[i++] = display_info->atoms[NET_CLIENT_LIST_STACKING];
@@ -370,6 +427,7 @@ setNetSupportedHint (DisplayInfo *display_info, Window root, Window check_win)
     atoms[i++] = display_info->atoms[NET_WM_ICON];
     atoms[i++] = display_info->atoms[NET_WM_ICON_GEOMETRY];
     atoms[i++] = display_info->atoms[NET_WM_ICON_NAME];
+    atoms[i++] = display_info->atoms[NET_WM_MOVERESIZE];
     atoms[i++] = display_info->atoms[NET_WM_NAME];
     atoms[i++] = display_info->atoms[NET_WM_STATE];
     atoms[i++] = display_info->atoms[NET_WM_STATE_ABOVE];
@@ -386,6 +444,8 @@ setNetSupportedHint (DisplayInfo *display_info, Window root, Window check_win)
     atoms[i++] = display_info->atoms[NET_WM_STATE_STICKY];
     atoms[i++] = display_info->atoms[NET_WM_STRUT];
     atoms[i++] = display_info->atoms[NET_WM_STRUT_PARTIAL];
+    atoms[i++] = display_info->atoms[NET_WM_SYNC_REQUEST];
+    atoms[i++] = display_info->atoms[NET_WM_SYNC_REQUEST_COUNTER];
     atoms[i++] = display_info->atoms[NET_WM_USER_TIME];
     atoms[i++] = display_info->atoms[NET_WM_WINDOW_TYPE];
     atoms[i++] = display_info->atoms[NET_WM_WINDOW_TYPE_DESKTOP];
@@ -400,11 +460,11 @@ setNetSupportedHint (DisplayInfo *display_info, Window root, Window check_win)
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
     atoms[i++] = display_info->atoms[NET_STARTUP_ID];
 #endif
-
+    g_assert (i < 64);
     data[0] = check_win;
-    XChangeProperty (display_info->dpy, root, display_info->atoms[NET_SUPPORTED], 
+    XChangeProperty (display_info->dpy, root, display_info->atoms[NET_SUPPORTED],
                      XA_ATOM, 32, PropModeReplace, (unsigned char *) atoms, i);
-    XChangeProperty (display_info->dpy, check_win, display_info->atoms[NET_SUPPORTING_WM_CHECK], 
+    XChangeProperty (display_info->dpy, check_win, display_info->atoms[NET_SUPPORTING_WM_CHECK],
                      XA_WINDOW, 32, PropModeReplace, (unsigned char *) data, 1);
     XChangeProperty (display_info->dpy, root, display_info->atoms[NET_SUPPORTING_WM_CHECK],
                      XA_WINDOW, 32, PropModeReplace, (unsigned char *) data, 1);
@@ -417,21 +477,22 @@ getAtomList (DisplayInfo *display_info, Window w, int atom_id, Atom ** atoms_p,
     int format;
     unsigned long n_atoms;
     unsigned long bytes_after;
+    unsigned char *data;
     Atom *atoms;
 
     *atoms_p = NULL;
     *n_atoms_p = 0;
 
-    g_return_val_if_fail (((atom_id > 0) && (atom_id < NB_ATOMS)), FALSE);
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
     TRACE ("entering getAtomList()");
 
-    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id], 
-                             0, G_MAXLONG, FALSE, XA_ATOM, &type, &format, &n_atoms, 
-                             &bytes_after, (unsigned char **) &atoms) != Success) || (type == None))
+    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id],
+                             0, G_MAXLONG, FALSE, XA_ATOM, &type, &format, &n_atoms,
+                             &bytes_after, (unsigned char **) &data) != Success) || (type == None))
     {
         return FALSE;
     }
-
+    atoms = (Atom *) data;
     if (!check_type_and_format (32, XA_ATOM, -1, format, type))
     {
         if (atoms)
@@ -456,22 +517,23 @@ getCardinalList (DisplayInfo *display_info, Window w, int atom_id, unsigned long
     int format;
     unsigned long n_cardinals;
     unsigned long bytes_after;
+    unsigned char *data;
     unsigned long *cardinals;
 
     *cardinals_p = NULL;
     *n_cardinals_p = 0;
 
-    g_return_val_if_fail (((atom_id > 0) && (atom_id < NB_ATOMS)), FALSE);
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
     TRACE ("entering getCardinalList()");
 
-    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id], 
+    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id],
                              0, G_MAXLONG, FALSE, XA_CARDINAL,
                              &type, &format, &n_cardinals, &bytes_after,
-                             (unsigned char **) &cardinals) != Success) || (type == None))
+                             (unsigned char **) &data) != Success) || (type == None))
     {
         return FALSE;
     }
-
+    cardinals = (unsigned long *) data;
     if (!check_type_and_format (32, XA_CARDINAL, -1, format, type))
     {
         XFree (cardinals);
@@ -496,12 +558,12 @@ setNetWorkarea (DisplayInfo *display_info, Window root, int nb_workspaces, int w
     ptr = data;
     for (i = 0; i < j; i++)
     {
-        *ptr++ = (unsigned long) m[LEFT];
-        *ptr++ = (unsigned long) m[TOP];
-        *ptr++ = (unsigned long) (width  - (m[LEFT] + m[RIGHT]));
-        *ptr++ = (unsigned long) (height - (m[TOP] + m[BOTTOM]));
+        *ptr++ = (unsigned long) m[STRUTS_LEFT];
+        *ptr++ = (unsigned long) m[STRUTS_TOP];
+        *ptr++ = (unsigned long) (width  - (m[STRUTS_LEFT] + m[STRUTS_RIGHT]));
+        *ptr++ = (unsigned long) (height - (m[STRUTS_TOP]  + m[STRUTS_BOTTOM]));
     }
-    XChangeProperty (display_info->dpy, root, display_info->atoms[NET_WORKAREA], 
+    XChangeProperty (display_info->dpy, root, display_info->atoms[NET_WORKAREA],
                      XA_CARDINAL, 32, PropModeReplace, (unsigned char *) data, j * 4);
     g_free (data);
 }
@@ -516,7 +578,7 @@ setNetFrameExtents (DisplayInfo *display_info, Window w, int top, int left, int
     data[1] = (unsigned long) right;
     data[2] = (unsigned long) top;
     data[3] = (unsigned long) bottom;
-    XChangeProperty (display_info->dpy, w, display_info->atoms[NET_FRAME_EXTENTS], 
+    XChangeProperty (display_info->dpy, w, display_info->atoms[NET_FRAME_EXTENTS],
                      XA_CARDINAL, 32, PropModeReplace, (unsigned char *) data, 4);
 }
 
@@ -539,12 +601,12 @@ initNetDesktopInfo (DisplayInfo *display_info, Window root, int workspace, int w
 }
 
 void
-setUTF8StringHint (DisplayInfo *display_info, Window w, int atom_id, const char *val)
+setUTF8StringHint (DisplayInfo *display_info, Window w, int atom_id, const gchar *val)
 {
-    g_return_if_fail ((atom_id > 0) && (atom_id < NB_ATOMS));
-    TRACE ("entering set_utf8_string_hint");
+    g_return_if_fail ((atom_id >= 0) && (atom_id < ATOM_COUNT));
+    TRACE ("entering setUTF8StringHint");
 
-    XChangeProperty (display_info->dpy, w, display_info->atoms[atom_id], 
+    XChangeProperty (display_info->dpy, w, display_info->atoms[atom_id],
                      display_info->atoms[UTF8_STRING], 8, PropModeReplace,
                      (unsigned char *) val, strlen (val) + 1);
 }
@@ -575,23 +637,6 @@ getTransientFor (DisplayInfo *display_info, Window root, Window w, Window * tran
     TRACE ("Window (0x%lx) is transient for (0x%lx)", w, *transient_for);
 }
 
-static gchar *
-internal_utf8_strndup (const gchar *src, gssize max_len)
-{
-    const gchar *s = src;
-
-    if (max_len <= 0)
-	return g_strdup (src);
-
-    while (max_len && *s)
-    {
-	s = g_utf8_next_char (s);
-	max_len--;
-    }
-
-    return g_strndup (src, s - src);
-}
-
 static char *
 text_property_to_utf8 (DisplayInfo *display_info, const XTextProperty * prop)
 {
@@ -602,7 +647,7 @@ text_property_to_utf8 (DisplayInfo *display_info, const XTextProperty * prop)
     TRACE ("entering text_property_to_utf8");
 
     list = NULL;
-    count = gdk_text_property_to_utf8_list (gdk_x11_xatom_to_atom (prop->encoding), 
+    count = gdk_text_property_to_utf8_list (gdk_x11_xatom_to_atom (prop->encoding),
                                             prop->format, prop->value, prop->nitems, &list);
     if (count == 0)
     {
@@ -646,7 +691,7 @@ get_text_property (DisplayInfo *display_info, Window w, Atom a)
 }
 
 static gboolean
-getUTF8StringData (DisplayInfo *display_info, Window w, int atom_id, char **str_p, int *length)
+getUTF8StringData (DisplayInfo *display_info, Window w, int atom_id, gchar **str_p, int *length)
 {
     Atom type;
     int format;
@@ -654,12 +699,12 @@ getUTF8StringData (DisplayInfo *display_info, Window w, int atom_id, char **str_
     unsigned char *str;
     unsigned long n_items;
 
-    g_return_val_if_fail (((atom_id > 0) && (atom_id < NB_ATOMS)), FALSE);
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
     TRACE ("entering getUTF8StringData");
 
     *str_p = NULL;
-    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id], 
-                             0, G_MAXLONG, FALSE, display_info->atoms[UTF8_STRING], &type, 
+    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id],
+                             0, G_MAXLONG, FALSE, display_info->atoms[UTF8_STRING], &type,
                              &format, &n_items, &bytes_after, (unsigned char **) &str) != Success) || (type == None))
     {
         TRACE ("no UTF8_STRING property found");
@@ -683,11 +728,11 @@ getUTF8StringData (DisplayInfo *display_info, Window w, int atom_id, char **str_
 }
 
 gboolean
-getUTF8String (DisplayInfo *display_info, Window w, int atom_id, char **str_p, int *length)
+getUTF8String (DisplayInfo *display_info, Window w, int atom_id, gchar **str_p, int *length)
 {
     char *xstr;
-    
-    g_return_val_if_fail (((atom_id > 0) && (atom_id < NB_ATOMS)), FALSE);
+
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
     TRACE ("entering getUTF8String");
 
     if (!getUTF8StringData (display_info, w, atom_id, &xstr, length))
@@ -701,7 +746,7 @@ getUTF8String (DisplayInfo *display_info, Window w, int atom_id, char **str_p, i
     /* gmalloc the returned string */
     *str_p = internal_utf8_strndup (xstr, MAX_STR_LENGTH);
     XFree (xstr);
-    
+
     if (!g_utf8_validate (*str_p, -1, NULL))
     {
         TRACE ("getUTF8String() returned invalid UTF-8 characters");
@@ -716,19 +761,19 @@ getUTF8String (DisplayInfo *display_info, Window w, int atom_id, char **str_p, i
     {
         xfce_utf8_remove_controls((gchar *) *str_p, -1, NULL);
     }
-    
+
     return TRUE;
 }
 
 gboolean
-getUTF8StringList (DisplayInfo *display_info, Window w, int atom_id, char ***str_p, int *n_items)
+getUTF8StringList (DisplayInfo *display_info, Window w, int atom_id, gchar ***str_p, int *n_items)
 {
     char *xstr, *ptr;
-    char **retval;
+    gchar **retval;
     guint i;
     int length;
 
-    g_return_val_if_fail (((atom_id > 0) && (atom_id < NB_ATOMS)), FALSE);
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
 
     TRACE ("entering getUTF8StringList");
 
@@ -739,7 +784,7 @@ getUTF8StringList (DisplayInfo *display_info, Window w, int atom_id, char ***str
     {
         return FALSE;
     }
-    
+
     i = 0;
     while (i < length)
     {
@@ -753,10 +798,10 @@ getUTF8StringList (DisplayInfo *display_info, Window w, int atom_id, char ***str
     {
         *n_items = *n_items + 1;
     }
- 
-    retval = g_new0 (char *, *n_items + 1);
+
+    retval = g_new0 (gchar *, *n_items + 1);
     ptr = xstr;
-    
+
     for (i = 0; i < *n_items; i++)
     {
         if (g_utf8_validate (ptr, -1, NULL))
@@ -771,106 +816,74 @@ getUTF8StringList (DisplayInfo *display_info, Window w, int atom_id, char ***str
         ptr += strlen (ptr) + 1;
     }
     XFree (xstr);
-    *str_p = retval;    
+    *str_p = retval;
 
     return TRUE;
 }
 
-void
-getWindowName (DisplayInfo *display_info, Window w, char **name)
+gboolean
+getClientMachine (DisplayInfo *display_info, Window w, gchar **machine)
 {
     char *str;
-    int len;
+    gboolean status;
 
-    TRACE ("entering getWindowName");
+    TRACE ("entering getClientMachine");
 
-    g_return_if_fail (name != NULL);
-    *name = NULL;
-    g_return_if_fail (w != None);
+    g_return_val_if_fail (machine != NULL, FALSE);
+    *machine = NULL;
+    g_return_val_if_fail (w != None, FALSE);
 
-    if (getUTF8StringData (display_info, w, NET_WM_NAME, &str, &len))
-    {
-        *name = internal_utf8_strndup (str, MAX_STR_LENGTH);
-        XFree (str);
-        return;
-    }
-    str = get_text_property (display_info, w, XA_WM_NAME);
+    status = FALSE;
+    str = get_text_property (display_info, w, display_info->atoms[WM_CLIENT_MACHINE]);
     if (str)
     {
-        *name = internal_utf8_strndup (str, MAX_STR_LENGTH);
+        *machine = g_strndup (str, MAX_STR_LENGTH);
         XFree (str);
+        status = TRUE;
     }
     else
     {
-        *name = strdup ("");
+        *machine = g_strdup ("");
     }
+    return status;
 }
 
 gboolean
-checkKdeSystrayWindow(DisplayInfo *display_info, Window window)
+getWindowName (DisplayInfo *display_info, Window w, gchar **title)
 {
-    Atom actual_type;
-    int actual_format;
-    unsigned long nitems;
-    unsigned long bytes_after;
-    Window trayIconForWindow;
+    char *str;
+    int len;
+    gchar *machine;
+    gchar *name;
+    gboolean status;
 
-    TRACE ("entering GetWindowRole");
-    g_return_val_if_fail (window != None, FALSE);
-    
-    XGetWindowProperty(display_info->dpy, window, display_info->atoms[KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR], 
-                       0L, sizeof(Window), FALSE, XA_WINDOW, &actual_type, &actual_format, 
-                       &nitems, &bytes_after, (unsigned char **)&trayIconForWindow);
+    TRACE ("entering getWindowName");
 
-    if ((actual_format == None) || 
-        (actual_type != XA_WINDOW) || 
-        (trayIconForWindow == None))
+    g_return_val_if_fail (title != NULL, FALSE);
+    *title = NULL;
+    g_return_val_if_fail (w != None, FALSE);
+
+    status = FALSE;
+    getClientMachine (display_info, w, &machine);
+    if (getUTF8StringData (display_info, w, NET_WM_NAME, &str, &len) ||
+        (str = get_text_property (display_info, w, XA_WM_NAME)))
     {
-        return FALSE;
+        name = internal_utf8_strndup (str, MAX_STR_LENGTH);
+        *title = create_name_with_host (display_info, name, machine);
+        g_free (name);
+        XFree (str);
+        status = TRUE;
     }
-    return TRUE;
-}
-
-void
-sendSystrayReqDock(DisplayInfo *display_info, Window window, Window systray)
-{
-    XClientMessageEvent xev;
-
-    TRACE ("entering sendSystrayReqDock");
-    g_return_if_fail (window != None);
-    g_return_if_fail (systray != None);
-
-    xev.type = ClientMessage;
-    xev.window = systray;
-    xev.message_type = display_info->atoms[NET_SYSTEM_TRAY_OPCODE];
-    xev.format = 32;
-    xev.data.l[0] = CurrentTime;
-    xev.data.l[1] = 0; /* SYSTEM_TRAY_REQUEST_DOCK */
-    xev.data.l[2] = window;
-    xev.data.l[3] = 0; /* Nada */
-    xev.data.l[4] = 0; /* Niet */
-
-    XSendEvent (display_info->dpy, systray, FALSE, NoEventMask, (XEvent *) & xev);
-}
-
-Window
-getSystrayWindow (DisplayInfo *display_info, Atom net_system_tray_selection)
-{
-    Window systray_win = None;
-
-    TRACE ("entering getSystrayWindow");
-
-    systray_win = XGetSelectionOwner (display_info->dpy, net_system_tray_selection);
-    if (systray_win)
+    else
     {
-        XSelectInput (display_info->dpy, systray_win, StructureNotifyMask);
+        *title = g_strdup ("");
     }
-    TRACE ("New systray window:  0x%lx", systray_win);
-    return systray_win;
+    g_free (machine);
+    return status;
 }
 
 gboolean
-getWindowRole (DisplayInfo *display_info, Window window, char **role)
+getWindowRole (DisplayInfo *display_info, Window window, gchar **role)
 {
     XTextProperty tp;
 
@@ -886,7 +899,7 @@ getWindowRole (DisplayInfo *display_info, Window window, char **role)
         {
             if ((tp.encoding == XA_STRING) && (tp.format == 8) && (tp.nitems != 0))
             {
-                *role = strdup ((char *) tp.value);
+                *role = g_strdup ((gchar *) tp.value);
                 XFree (tp.value);
                 return TRUE;
             }
@@ -900,20 +913,21 @@ getWindowRole (DisplayInfo *display_info, Window window, char **role)
 Window
 getClientLeader (DisplayInfo *display_info, Window window)
 {
-    Window client_leader = None;
+    Window client_leader;
     Atom actual_type;
     int actual_format;
     unsigned long nitems;
     unsigned long bytes_after;
-    unsigned char *prop = NULL;
+    unsigned char *prop;
 
     TRACE ("entering getClientLeader");
 
     g_return_val_if_fail (window != None, None);
 
-    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[WM_CLIENT_LEADER], 
+    client_leader = None;
+    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[WM_CLIENT_LEADER],
                             0L, 1L, FALSE, AnyPropertyType, &actual_type, &actual_format, &nitems,
-                            &bytes_after, &prop) == Success)
+                            &bytes_after, (unsigned char **) &prop) == Success)
     {
         if ((prop) && (actual_type == XA_WINDOW) && (actual_format == 32)
             && (nitems == 1) && (bytes_after == 0))
@@ -941,9 +955,9 @@ getNetWMUserTime (DisplayInfo *display_info, Window window, Time *time)
 
     g_return_val_if_fail (window != None, None);
 
-    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[NET_WM_USER_TIME], 
+    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[NET_WM_USER_TIME],
                             0L, 1L, FALSE, XA_CARDINAL, &actual_type, &actual_format, &nitems,
-                            &bytes_after, &data) == Success)
+                            &bytes_after, (unsigned char **) &data) == Success)
     {
         if ((data) && (actual_type == XA_CARDINAL)
             && (nitems == 1) && (bytes_after == 0))
@@ -958,7 +972,7 @@ getNetWMUserTime (DisplayInfo *display_info, Window window, Time *time)
 }
 
 gboolean
-getClientID (DisplayInfo *display_info, Window window, char **client_id)
+getClientID (DisplayInfo *display_info, Window window, gchar **client_id)
 {
     Window id;
     XTextProperty tp;
@@ -975,7 +989,7 @@ getClientID (DisplayInfo *display_info, Window window, char **client_id)
         {
             if (tp.encoding == XA_STRING && tp.format == 8 && tp.nitems != 0)
             {
-                *client_id = strdup ((char *) tp.value);
+                *client_id = g_strdup ((gchar *) tp.value);
                 XFree (tp.value);
                 return TRUE;
             }
@@ -1014,19 +1028,20 @@ getKDEIcon (DisplayInfo *display_info, Window window, Pixmap * pixmap, Pixmap *
     int format;
     unsigned long nitems;
     unsigned long bytes_after;
+    unsigned char *data;
     Pixmap *icons;
 
     *pixmap = None;
     *mask = None;
 
     icons = NULL;
-    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[KWM_WIN_ICON], 
-                            0L, G_MAXLONG, FALSE, display_info->atoms[KWM_WIN_ICON], &type, 
-                            &format, &nitems, &bytes_after, (unsigned char **)&icons) != Success)
+    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[KWM_WIN_ICON],
+                            0L, G_MAXLONG, FALSE, display_info->atoms[KWM_WIN_ICON], &type,
+                            &format, &nitems, &bytes_after, (unsigned char **)&data) != Success)
     {
         return FALSE;
     }
-
+    icons = (Pixmap *) data;
     if (type != display_info->atoms[KWM_WIN_ICON])
     {
         if (icons)
@@ -1051,14 +1066,14 @@ getRGBIconData (DisplayInfo *display_info, Window window, unsigned long **data,
     int format;
     unsigned long bytes_after;
 
-    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[NET_WM_ICON], 
+    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[NET_WM_ICON],
                             0L, G_MAXLONG, FALSE, XA_CARDINAL, &type, &format, nitems,
-                            &bytes_after, (unsigned char **)data) != Success)
+                            &bytes_after, (unsigned char **) data) != Success)
     {
         *data = NULL;
         return FALSE;
     }
-    
+
     if (type != XA_CARDINAL)
     {
         if (*data)
@@ -1075,12 +1090,13 @@ getRGBIconData (DisplayInfo *display_info, Window window, unsigned long **data,
 gboolean
 getOpacity (DisplayInfo *display_info, Window window, guint *opacity)
 {
-    long val = 0;
-    
+    long val;
+
     g_return_val_if_fail (window != None, FALSE);
     g_return_val_if_fail (opacity != NULL, FALSE);
-    TRACE ("entering getOpacity");    
+    TRACE ("entering getOpacity");
 
+    val = 0;
     if (getHint (display_info, window, NET_WM_WINDOW_OPACITY, &val))
     {
         *opacity = (guint) val;
@@ -1090,9 +1106,180 @@ getOpacity (DisplayInfo *display_info, Window window, guint *opacity)
     return FALSE;
 }
 
+gboolean
+getOpacityLock (DisplayInfo *display_info, Window window)
+{
+    long val;
+
+    g_return_val_if_fail (window != None, FALSE);
+    TRACE ("entering getOpacityLock");
+
+    /* only presence/absence matters */
+    return !!getHint (display_info, window, NET_WM_WINDOW_OPACITY_LOCKED, &val);
+}
+
+gboolean
+setXAtomManagerOwner (DisplayInfo *display_info, Atom atom, Window root, Window w)
+{
+    XClientMessageEvent ev;
+    Time server_time;
+    int status;
+
+    g_return_val_if_fail (root != None, FALSE);
+
+    server_time = myDisplayGetCurrentTime (display_info);
+    status = XSetSelectionOwner (display_info->dpy, atom, w, server_time);
+
+    if ((status == BadAtom) || (status == BadWindow))
+    {
+        return FALSE;
+    }
+
+    if (XGetSelectionOwner (display_info->dpy, atom) == w)
+    {
+        ev.type = ClientMessage;
+        ev.message_type = atom;
+        ev.format = 32;
+        ev.data.l[0] = server_time;
+        ev.data.l[1] = atom;
+        ev.data.l[2] = w;
+        ev.data.l[3] = 0;
+        ev.data.l[4] = 0;
+        ev.window = root;
+
+        XSendEvent (display_info->dpy, root, FALSE, StructureNotifyMask, (XEvent *) &ev);
+
+        return TRUE;
+    }
+
+    return FALSE;
+}
+
+gboolean
+setAtomIdManagerOwner (DisplayInfo *display_info, int atom_id, Window root, Window w)
+{
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
+
+    return setXAtomManagerOwner(display_info, display_info->atoms[atom_id], root, w);
+}
+
+
+static Bool
+checkPropEvent (Display *display, XEvent *xevent, XPointer arg)
+{
+    DisplayInfo *display_info;
+    ScreenInfo *screen_info;
+
+    display_info = (DisplayInfo *) arg;
+    g_return_val_if_fail (display_info, FALSE);
+
+    screen_info = myDisplayGetDefaultScreen (display_info);
+    g_return_val_if_fail (screen_info, FALSE);
+
+    if ((xevent->type == PropertyNotify) &&
+        (xevent->xproperty.window == screen_info->xfwm4_win) &&
+        (xevent->xproperty.atom == display_info->atoms[XFWM4_TIMESTAMP_PROP]))
+    {
+        return TRUE;
+    }
+
+    return FALSE;
+}
+
+Time
+getXServerTime (DisplayInfo *display_info)
+{
+    ScreenInfo *screen_info;
+    XEvent xevent;
+    Time timestamp;
+    char c = '\0';
+
+    g_return_val_if_fail (display_info, (Time) CurrentTime);
+    timestamp = myDisplayGetCurrentTime (display_info);
+    if (timestamp == CurrentTime)
+    {
+        screen_info = myDisplayGetDefaultScreen (display_info);
+        g_return_val_if_fail (screen_info,  (Time) CurrentTime);
+
+        XChangeProperty (display_info->dpy, screen_info->xfwm4_win, 
+                         display_info->atoms[XFWM4_TIMESTAMP_PROP],
+                         display_info->atoms[XFWM4_TIMESTAMP_PROP],
+                         8, PropModeReplace, (unsigned char *) &c, 1);
+        XIfEvent (display_info->dpy, &xevent, checkPropEvent, (XPointer) display_info);
+
+        timestamp = (Time) xevent.xproperty.time;
+    }
+    return timestamp;
+}
+
+#ifdef ENABLE_KDE_SYSTRAY_PROXY
+gboolean
+checkKdeSystrayWindow(DisplayInfo *display_info, Window window)
+{
+    Atom actual_type;
+    int actual_format;
+    unsigned long nitems;
+    unsigned long bytes_after;
+    Window trayIconForWindow;
+
+    TRACE ("entering GetWindowRole");
+    g_return_val_if_fail (window != None, FALSE);
+
+    XGetWindowProperty(display_info->dpy, window, display_info->atoms[KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR],
+                       0L, sizeof(Window), FALSE, XA_WINDOW, &actual_type, &actual_format,
+                       &nitems, &bytes_after, (unsigned char **) &trayIconForWindow);
+
+    if ((actual_format == None) ||
+        (actual_type != XA_WINDOW) ||
+        (trayIconForWindow == None))
+    {
+        return FALSE;
+    }
+    return TRUE;
+}
+
+void
+sendSystrayReqDock(DisplayInfo *display_info, Window window, Window systray)
+{
+    XClientMessageEvent xev;
+
+    TRACE ("entering sendSystrayReqDock");
+    g_return_if_fail (window != None);
+    g_return_if_fail (systray != None);
+
+    xev.type = ClientMessage;
+    xev.window = systray;
+    xev.message_type = display_info->atoms[NET_SYSTEM_TRAY_OPCODE];
+    xev.format = 32;
+    xev.data.l[0] = myDisplayGetCurrentTime (screen_info);
+    xev.data.l[1] = 0; /* SYSTEM_TRAY_REQUEST_DOCK */
+    xev.data.l[2] = window;
+    xev.data.l[3] = 0; /* Nada */
+    xev.data.l[4] = 0; /* Niet */
+
+    XSendEvent (display_info->dpy, systray, FALSE, NoEventMask, (XEvent *) & xev);
+}
+
+Window
+getSystrayWindow (DisplayInfo *display_info, Atom net_system_tray_selection)
+{
+    Window systray_win;
+
+    TRACE ("entering getSystrayWindow");
+
+    systray_win = XGetSelectionOwner (display_info->dpy, net_system_tray_selection);
+    if (systray_win)
+    {
+        XSelectInput (display_info->dpy, systray_win, StructureNotifyMask);
+    }
+    TRACE ("New systray window:  0x%lx", systray_win);
+    return systray_win;
+}
+#endif
+
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
 gboolean
-getWindowStartupId (DisplayInfo *display_info, Window w, char **startup_id)
+getWindowStartupId (DisplayInfo *display_info, Window w, gchar **startup_id)
 {
     char *str;
     int len;
@@ -1105,7 +1292,7 @@ getWindowStartupId (DisplayInfo *display_info, Window w, char **startup_id)
 
     if (getUTF8StringData (display_info, w, NET_STARTUP_ID, &str, &len))
     {
-        *startup_id = strdup (str);
+        *startup_id = g_strdup (str);
         XFree (str);
         return TRUE;
     }
@@ -1113,7 +1300,7 @@ getWindowStartupId (DisplayInfo *display_info, Window w, char **startup_id)
     str = get_text_property (display_info, w, NET_STARTUP_ID);
     if (str)
     {
-        *startup_id = strdup (str);
+        *startup_id = g_strdup (str);
         XFree (str);
         return TRUE;
     }
@@ -1121,3 +1308,44 @@ getWindowStartupId (DisplayInfo *display_info, Window w, char **startup_id)
     return FALSE;
 }
 #endif
+
+#ifdef HAVE_XSYNC
+gboolean
+getXSyncCounter (DisplayInfo *display_info, Window window, XSyncCounter *counter)
+{
+    long val;
+
+    g_return_val_if_fail (window != None, FALSE);
+    g_return_val_if_fail (counter != NULL, FALSE);
+    TRACE ("entering getXSyncCounter");
+
+    val = 0;
+    if (getHint (display_info, window, NET_WM_SYNC_REQUEST_COUNTER, &val))
+    {
+        *counter = (XSyncCounter) val;
+        return TRUE;
+    }
+
+    return FALSE;
+}
+
+void
+sendXSyncRequest (DisplayInfo *display_info, Window window, XSyncValue value)
+{
+    XClientMessageEvent xev;
+
+    g_return_if_fail (window != None);
+    TRACE ("entering getXSyncCounter");
+
+    xev.type = ClientMessage;
+    xev.window = window;
+    xev.message_type = display_info->atoms[WM_PROTOCOLS];
+    xev.format = 32;
+    xev.data.l[0] = display_info->atoms[NET_WM_SYNC_REQUEST];
+    xev.data.l[1] = CurrentTime;
+    xev.data.l[2] = XSyncValueLow32 (value);
+    xev.data.l[3] = XSyncValueHigh32 (value);
+    xev.data.l[4] = 0;
+    XSendEvent (display_info->dpy, window, FALSE, NoEventMask, (XEvent *) &xev);
+}
+#endif /* HAVE_XSYNC */
