@@ -1,23 +1,23 @@
-/*      $Id: netwm.c 20325 2006-03-10 23:36:36Z olivier $
- 
+/*      $Id: netwm.c 23704 2006-11-04 22:36:42Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
- 
+
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
  */
- 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -29,25 +29,25 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <libxfce4util/libxfce4util.h> 
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4util/libxfce4util.h>
 
-#include "display.h"
-#include "screen.h"
-#include "netwm.h"
-#include "misc.h"
 #include "client.h"
+#include "compositor.h"
+#include "display.h"
 #include "frame.h"
 #include "hints.h"
-#include "workspaces.h"
-#include "transients.h"
+#include "misc.h"
+#include "netwm.h"
+#include "screen.h"
 #include "stacking.h"
+#include "transients.h"
+#include "workspaces.h"
 
 void
 clientSetNetState (Client * c)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
     Atom data[16];
     int i;
 
@@ -125,11 +125,11 @@ clientSetNetState (Client * c)
         TRACE ("clientSetNetState : demands_attention");
         data[i++] = display_info->atoms[NET_WM_STATE_DEMANDS_ATTENTION];
     }
-    XChangeProperty (display_info->dpy, c->window, 
+    XChangeProperty (display_info->dpy, c->window,
                      display_info->atoms[NET_WM_STATE], XA_ATOM, 32,
                      PropModeReplace, (unsigned char *) data, i);
     /*
-       We also set GNOME hint here for consistency and convenience, 
+       We also set GNOME hint here for consistency and convenience,
        although the meaning of net_wm_state and win_state aren't the same.
      */
     setHint (display_info, c->window, WIN_STATE, c->win_state);
@@ -138,10 +138,10 @@ clientSetNetState (Client * c)
 void
 clientGetNetState (Client * c)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
-    int n_atoms = 0;
-    Atom *atoms = NULL;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+    int n_atoms;
+    Atom *atoms;
 
     g_return_if_fail (c != NULL);
     TRACE ("entering clientGetNetState");
@@ -149,6 +149,8 @@ clientGetNetState (Client * c)
 
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
+    atoms = NULL;
+    n_atoms = 0;
 
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_SESSION_MANAGED))
     {
@@ -177,7 +179,7 @@ clientGetNetState (Client * c)
             FLAG_SET (c->flags, CLIENT_FLAG_MAXIMIZED_VERT);
         }
     }
-    
+
     if (getAtomList (display_info, c->window, NET_WM_STATE, &atoms, &n_atoms))
     {
         int i;
@@ -276,9 +278,9 @@ clientGetNetState (Client * c)
 void
 clientUpdateNetState (Client * c, XClientMessageEvent * ev)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
-    unsigned long action;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+    unsigned long action, mode;
     Atom first;
     Atom second;
 
@@ -289,11 +291,12 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
 
-    action = ((XEvent *) ev)->xclient.data.l[0];
-    first  = ((XEvent *) ev)->xclient.data.l[1];
-    second = ((XEvent *) ev)->xclient.data.l[2];
+    action = ev->data.l[0];
+    first  = ev->data.l[1];
+    second = ev->data.l[2];
+    mode = 0;
 
-    if ((first  == display_info->atoms[NET_WM_STATE_SHADED]) || 
+    if ((first  == display_info->atoms[NET_WM_STATE_SHADED]) ||
         (second == display_info->atoms[NET_WM_STATE_SHADED]))
     {
         if ((action == NET_WM_STATE_ADD) && !FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
@@ -310,7 +313,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
         }
     }
 
-    if ((first  == display_info->atoms[NET_WM_STATE_STICKY]) || 
+    if ((first  == display_info->atoms[NET_WM_STATE_STICKY]) ||
         (second == display_info->atoms[NET_WM_STATE_STICKY]))
     {
         if (!clientIsValidTransientOrModal (c) && FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_STICK))
@@ -327,11 +330,11 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
             {
                 clientToggleSticky (c, TRUE);
             }
-            frameDraw (c, FALSE, FALSE);
+            frameDraw (c, FALSE);
         }
     }
 
-    if ((first  == display_info->atoms[NET_WM_STATE_DEMANDS_ATTENTION]) || 
+    if ((first  == display_info->atoms[NET_WM_STATE_DEMANDS_ATTENTION]) ||
         (second == display_info->atoms[NET_WM_STATE_DEMANDS_ATTENTION]))
     {
         if (action == NET_WM_STATE_ADD)
@@ -348,7 +351,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
         }
     }
 
-    if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) || 
+    if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) ||
         (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) ||
         (first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]) ||
         (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]))
@@ -357,7 +360,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
         {
             if ((action == NET_WM_STATE_ADD) && !FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED))
             {
-                unsigned long mode = 0;
+                mode = 0;
                 if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]))
                 {
@@ -368,11 +371,11 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
                 {
                     mode |= !FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_VERT) ? WIN_STATE_MAXIMIZED_VERT : 0;
                 }
-                clientToggleMaximized (c, mode);
+                clientToggleMaximized (c, mode, TRUE);
             }
             else if ((action == NET_WM_STATE_REMOVE) && FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED))
             {
-                unsigned long mode = 0;
+                mode = 0;
                 if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]))
                 {
@@ -383,27 +386,27 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
                 {
                     mode |= FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_VERT) ? WIN_STATE_MAXIMIZED_VERT : 0;
                 }
-                clientToggleMaximized (c, mode);
+                clientToggleMaximized (c, mode, TRUE);
             }
             else if (action == NET_WM_STATE_TOGGLE)
             {
-                unsigned long mode = 0;
+                mode = 0;
                 if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]))
                 {
                     mode |= WIN_STATE_MAXIMIZED_HORIZ;
                 }
-                if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]) || 
+                if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]))
                 {
                     mode |= WIN_STATE_MAXIMIZED_VERT;
                 }
-                clientToggleMaximized (c, mode);
+                clientToggleMaximized (c, mode, TRUE);
             }
         }
     }
 
-    if ((first  == display_info->atoms[NET_WM_STATE_MODAL]) || 
+    if ((first  == display_info->atoms[NET_WM_STATE_MODAL]) ||
         (second == display_info->atoms[NET_WM_STATE_MODAL]))
     {
         if ((action == NET_WM_STATE_ADD) && !FLAG_TEST (c->flags, CLIENT_FLAG_STATE_MODAL))
@@ -424,7 +427,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
             clientSetNetState (c);
             clientWindowType (c);
         }
-        frameDraw (c, TRUE, FALSE);
+        frameDraw (c, TRUE);
     }
 
     if ((first  == display_info->atoms[NET_WM_STATE_FULLSCREEN]) ||
@@ -450,7 +453,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
         }
     }
 
-    if ((first  == display_info->atoms[NET_WM_STATE_ABOVE]) || 
+    if ((first  == display_info->atoms[NET_WM_STATE_ABOVE]) ||
         (second == display_info->atoms[NET_WM_STATE_ABOVE]))
     {
         if (!FLAG_TEST (c->flags, CLIENT_FLAG_BELOW))
@@ -473,23 +476,26 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
         }
     }
 
-    if ((first  == display_info->atoms[NET_WM_STATE_BELOW]) || 
+    if ((first  == display_info->atoms[NET_WM_STATE_BELOW]) ||
         (second == display_info->atoms[NET_WM_STATE_BELOW]))
     {
         if (!FLAG_TEST (c->flags, CLIENT_FLAG_ABOVE))
         {
             if ((action == NET_WM_STATE_ADD) && !FLAG_TEST (c->flags, CLIENT_FLAG_BELOW))
             {
+                compositorDamageWindow (display_info, c->frame);
                 FLAG_SET (c->flags, CLIENT_FLAG_BELOW);
                 clientUpdateBelowState (c);
             }
             else if ((action == NET_WM_STATE_REMOVE) && FLAG_TEST (c->flags, CLIENT_FLAG_BELOW))
             {
+                compositorDamageWindow (display_info, c->frame);
                 FLAG_UNSET (c->flags, CLIENT_FLAG_BELOW);
                 clientUpdateBelowState (c);
             }
             else if (action == NET_WM_STATE_TOGGLE)
             {
+                compositorDamageWindow (display_info, c->frame);
                 FLAG_TOGGLE (c->flags, CLIENT_FLAG_BELOW);
                 clientUpdateBelowState (c);
             }
@@ -534,13 +540,145 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
             FLAG_TOGGLE (c->flags, CLIENT_FLAG_SKIP_TASKBAR);
             clientSetNetState (c);
         }
-        frameDraw (c, TRUE, FALSE);
+        frameDraw (c, TRUE);
+    }
+}
+
+void
+clientNetMoveResize (Client * c, XClientMessageEvent * ev)
+{
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+    unsigned int button_mask;
+    int x_root, y_root, dx, dy, action, button;
+    int corner;
+    gboolean resize; /* true == resize, false == move */
+    XEvent *event;
+
+    g_return_if_fail (c != NULL);
+    TRACE ("entering clientNetMoveResize");
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
+
+    screen_info = c->screen_info;
+    display_info = screen_info->display_info;
+
+    x_root = (int) ev->data.l[0];
+    y_root = (int) ev->data.l[1];
+    action = (int) ev->data.l[2];
+    button = (int) ev->data.l[3];
+    event  = (XEvent *) ev;
+
+    if (button == 0)
+    {
+        button_mask = getMouseXY (screen_info, c->window, &dx, &dy);
+        if (button_mask & Button1Mask)
+        {
+            button = Button1;
+        }
+        else if (button_mask & Button2Mask)
+        {
+            button = Button2;
+        }
+        else if (button_mask & Button3Mask)
+        {
+            button = Button3;
+        }
+        else
+        {
+            g_warning ("Could not determine the mouse button used");
+            return;
+        }
+    }
+
+    corner = CORNER_BOTTOM_RIGHT;
+    resize = TRUE;
+
+    event->xbutton.button = button;
+    event->xbutton.x_root = event->xkey.x_root = x_root;
+    event->xbutton.y_root = event->xkey.y_root = y_root;
+    event->xbutton.time = event->xkey.time = myDisplayGetCurrentTime (display_info);
+
+    switch (action)
+    {
+        /* Keyboard */
+        case NET_WM_MOVERESIZE_SIZE_KEYBOARD:
+            event->type = KeyPress;
+            corner = CORNER_BOTTOM_RIGHT;
+            resize = TRUE; /* Resize */
+            break;
+        case NET_WM_MOVERESIZE_MOVE_KEYBOARD:
+            event->type = KeyPress;
+            resize = FALSE; /* Move */
+            break;
+
+        /* Sides */
+        case NET_WM_MOVERESIZE_SIZE_TOP:
+            event->type = ButtonPress;
+            corner = CORNER_COUNT + SIDE_TOP;
+            resize = TRUE; /* Resize */
+            break;
+        case NET_WM_MOVERESIZE_SIZE_BOTTOM:
+            event->type = ButtonPress;
+            corner = CORNER_COUNT + SIDE_BOTTOM;
+            resize = TRUE; /* Resize */
+            break;
+        case NET_WM_MOVERESIZE_SIZE_RIGHT:
+            event->type = ButtonPress;
+            corner = CORNER_COUNT + SIDE_RIGHT;
+            resize = TRUE; /* Resize */
+            break;
+        case NET_WM_MOVERESIZE_SIZE_LEFT:
+            event->type = ButtonPress;
+            corner = CORNER_COUNT + SIDE_LEFT;
+            resize = TRUE; /* Resize */
+            break;
+
+        /* Corners */
+        case NET_WM_MOVERESIZE_SIZE_TOPLEFT:
+            event->type = ButtonPress;
+            corner = CORNER_TOP_LEFT;
+            resize = TRUE; /* Resize */
+            break;
+        case NET_WM_MOVERESIZE_SIZE_TOPRIGHT:
+            event->type = ButtonPress;
+            corner = CORNER_TOP_RIGHT;
+            resize = TRUE; /* Resize */
+            break;
+        case NET_WM_MOVERESIZE_SIZE_BOTTOMLEFT:
+            event->type = ButtonPress;
+            corner = CORNER_BOTTOM_LEFT;
+            resize = TRUE; /* Resize */
+            break;
+        case NET_WM_MOVERESIZE_SIZE_BOTTOMRIGHT:
+            event->type = ButtonPress;
+            corner = CORNER_BOTTOM_RIGHT;
+            resize = TRUE; /* Resize */
+            break;
+        case NET_WM_MOVERESIZE_MOVE:
+        default:
+            event->type = ButtonPress;
+            resize = FALSE; /* Move */
+            break;
+    }
+    
+    if (!FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
+    {
+        if (resize && FLAG_TEST_ALL (c->xfwm_flags, XFWM_FLAG_HAS_RESIZE | XFWM_FLAG_IS_RESIZABLE))
+        {
+            clientResize (c, corner, event);
+        }
+        else if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_MOVE))
+        {
+            clientMove (c, event);
+        }
     }
 }
 
 void
 clientUpdateFullscreenState (Client * c)
 {
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
     XWindowChanges wc;
     int layer;
 
@@ -548,6 +686,9 @@ clientUpdateFullscreenState (Client * c)
     TRACE ("entering clientUpdateFullscreenState");
     TRACE ("Update fullscreen state for client \"%s\" (0x%lx)", c->name, c->window);
 
+    screen_info = c->screen_info;
+    display_info = screen_info->display_info;
+
     if (FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
     {
         GdkRectangle rect;
@@ -583,15 +724,15 @@ clientUpdateFullscreenState (Client * c)
     clientSetNetState (c);
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_MANAGED))
     {
-        /* 
+        /*
            For some reason, the configure can generate EnterNotify events
            on lower windows, causing a nasty race cond with apps trying to
            grab focus in focus follow mouse mode. Grab the pointer to
            avoid these effects
          */
-        myScreenGrabPointer (c->screen_info, EnterWindowMask, None, CurrentTime);
+        myScreenGrabPointer (c->screen_info, EnterWindowMask, None, myDisplayGetCurrentTime (display_info));
         clientConfigure (c, &wc, CWX | CWY | CWWidth | CWHeight, CFG_FORCE_REDRAW);
-        myScreenUngrabPointer (c->screen_info, CurrentTime);
+        myScreenUngrabPointer (c->screen_info, myDisplayGetCurrentTime (display_info));
     }
     else
     {
@@ -606,10 +747,10 @@ clientUpdateFullscreenState (Client * c)
 void
 clientGetNetWmType (Client * c)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
-    int n_atoms = 0;
-    Atom *atoms = NULL;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+    int n_atoms;
+    Atom *atoms;
     int i;
 
     g_return_if_fail (c != NULL);
@@ -618,7 +759,6 @@ clientGetNetWmType (Client * c)
 
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
-
     n_atoms = 0;
     atoms = NULL;
 
@@ -645,13 +785,13 @@ clientGetNetWmType (Client * c)
         i = 0;
         while (i < n_atoms)
         {
-            if ((atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_DESKTOP]) || 
-                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_DOCK])    || 
-                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_TOOLBAR]) || 
-                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_MENU])    || 
-                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_DIALOG])  || 
-                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_NORMAL])  || 
-                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_UTILITY]) || 
+            if ((atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_DESKTOP]) ||
+                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_DOCK])    ||
+                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_TOOLBAR]) ||
+                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_MENU])    ||
+                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_DIALOG])  ||
+                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_NORMAL])  ||
+                (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_UTILITY]) ||
                 (atoms[i] == display_info->atoms[NET_WM_WINDOW_TYPE_SPLASH]))
             {
                 c->type_atom = atoms[i];
@@ -670,10 +810,10 @@ clientGetNetWmType (Client * c)
 void
 clientGetInitialNetWmDesktop (Client * c)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
-    Client *c2 = NULL;
-    long val = 0;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+    Client *c2;
+    long val;
 
     g_return_if_fail (c != NULL);
     TRACE ("entering clientGetInitialNetWmDesktop");
@@ -681,6 +821,7 @@ clientGetInitialNetWmDesktop (Client * c)
 
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
+    val = 0;
 
     /* This is to make sure that transient are shown with their "ancestor" window */
     c2 = clientGetTransient (c);
@@ -762,8 +903,11 @@ clientSetNetClientList (ScreenInfo * screen_info, Atom a, GList * list)
     if (size < 1)
     {
         XDeleteProperty (myScreenGetXDisplay (screen_info), screen_info->xroot, a);
+        return;
     }
-    else if ((listw = (Window *) malloc ((size + 1) * sizeof (Window))))
+
+    listw = g_new (Window, size + 1);
+    if (listw)
     {
         TRACE ("%i windows in list for %i clients", size, screen_info->client_count);
         for (i = 0, index_dest = listw, index_src = list; i < size;
@@ -774,16 +918,16 @@ clientSetNetClientList (ScreenInfo * screen_info, Atom a, GList * list)
         }
         XChangeProperty (myScreenGetXDisplay (screen_info), screen_info->xroot, a, XA_WINDOW, 32, PropModeReplace,
             (unsigned char *) listw, size);
-        free (listw);
+        g_free (listw);
     }
 }
 
 void
 clientGetNetStruts (Client * c)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
-    gulong *struts = NULL;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+    gulong *struts;
     int nitems;
     int i;
 
@@ -792,6 +936,7 @@ clientGetNetStruts (Client * c)
 
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
+    struts = NULL;
 
     for (i = 0; i < 12; i++)
     {
@@ -842,13 +987,13 @@ clientGetNetStruts (Client * c)
             c->struts[i] = 0;
         }
         /* Fill(in values as for partial struts */
-        c->struts[TOP_START_X] = c->struts[BOTTOM_START_X] = 0;
-        c->struts[TOP_END_X] = c->struts[BOTTOM_END_X] = 
-            gdk_screen_get_width (c->screen_info->gscr);
-        c->struts[LEFT_START_Y] = c->struts[RIGHT_START_Y] = 0;
-        c->struts[LEFT_END_Y] = c->struts[RIGHT_END_Y] = 
-            gdk_screen_get_height (c->screen_info->gscr);
-        
+        c->struts[STRUTS_TOP_START_X] = c->struts[STRUTS_BOTTOM_START_X] = 0;
+        c->struts[STRUTS_TOP_END_X] = c->struts[STRUTS_BOTTOM_END_X] =
+            c->screen_info->width;
+        c->struts[STRUTS_LEFT_START_Y] = c->struts[STRUTS_RIGHT_START_Y] = 0;
+        c->struts[STRUTS_LEFT_END_Y] = c->struts[STRUTS_RIGHT_END_Y] =
+            c->screen_info->height;
+
         XFree (struts);
         workspaceUpdateArea (c->screen_info);
     }
@@ -857,16 +1002,17 @@ clientGetNetStruts (Client * c)
 void
 clientSetNetActions (Client * c)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
     Atom atoms[6];
-    int i = 0;
+    int i;
 
     g_return_if_fail (c != NULL);
     TRACE ("entering clientSetNetActions");
 
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
+    i = 0;
 
     atoms[i++] = display_info->atoms[NET_WM_ACTION_CLOSE];
     if (CLIENT_CAN_MAXIMIZE_WINDOW (c))
@@ -883,15 +1029,15 @@ clientSetNetActions (Client * c)
     {
         atoms[i++] = display_info->atoms[NET_WM_ACTION_SHADE];
     }
-    XChangeProperty (clientGetXDisplay (c), c->window, display_info->atoms[NET_WM_ALLOWED_ACTIONS], 
+    XChangeProperty (clientGetXDisplay (c), c->window, display_info->atoms[NET_WM_ALLOWED_ACTIONS],
                      XA_ATOM, 32, PropModeReplace, (unsigned char *) atoms, i);
 }
 
 void
 clientWindowType (Client * c)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
     netWindowType old_type;
 
     g_return_if_fail (c != NULL);
@@ -915,8 +1061,8 @@ clientWindowType (Client * c)
                 CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_STICKY |
                 CLIENT_FLAG_SKIP_TASKBAR);
             FLAG_UNSET (c->xfwm_flags,
-                XFWM_FLAG_HAS_RESIZE | XFWM_FLAG_HAS_MOVE | 
-                XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE | 
+                XFWM_FLAG_HAS_RESIZE | XFWM_FLAG_HAS_MOVE |
+                XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE |
                 XFWM_FLAG_HAS_MENU | XFWM_FLAG_HAS_STICK |
                 XFWM_FLAG_HAS_BORDER);
         }
@@ -929,7 +1075,7 @@ clientWindowType (Client * c)
                 CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_SKIP_TASKBAR);
             FLAG_UNSET (c->xfwm_flags,
                 XFWM_FLAG_HAS_BORDER |  XFWM_FLAG_HAS_MOVE |
-                XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE | 
+                XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE |
                 XFWM_FLAG_HAS_MENU | XFWM_FLAG_HAS_STICK);
         }
         else if (c->type_atom == display_info->atoms[NET_WM_WINDOW_TYPE_TOOLBAR])
