@@ -1,47 +1,45 @@
-/*      $Id: transients.c,v 1.16 2004/12/30 22:08:37 olivier Exp $
- 
+/*      $Id: transients.c 23128 2006-09-11 20:39:40Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
- 
+
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
  */
- 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <glib.h>
-#include <libxfce4util/libxfce4util.h> 
+#include <libxfce4util/libxfce4util.h>
 
 #include "screen.h"
 #include "client.h"
 #include "stacking.h"
+#include "transients.h"
 
 Client *
 clientGetTransient (Client * c)
 {
-    Client *c2 = NULL;
-
     g_return_val_if_fail (c != NULL, NULL);
 
     TRACE ("entering clientGetTransient");
 
     if ((c->transient_for) && (c->transient_for != c->screen_info->xroot))
     {
-        c2 = clientGetFromWindow (c->screen_info, c->transient_for, WINDOW);
-        return c2;
+        return clientGetFromWindow (c->screen_info, c->transient_for, WINDOW);
     }
     return NULL;
 }
@@ -53,7 +51,7 @@ clientIsTransient (Client * c)
 
     TRACE ("entering clientIsTransient");
 
-    return (((c->transient_for != c->screen_info->xroot) && (c->transient_for != None) && (c->transient_for != c->window)) || 
+    return (((c->transient_for != c->screen_info->xroot) && (c->transient_for != None) && (c->transient_for != c->window)) ||
             ((c->transient_for == c->screen_info->xroot) && (c->group_leader != None) && (c->group_leader != c->window)));
 }
 
@@ -87,8 +85,8 @@ clientSameGroup (Client * c1, Client * c2)
 
     TRACE ("entering clientSameGroup");
 
-    return ((c1 != c2) && 
-            (((c1->group_leader != None) && 
+    return ((c1 != c2) &&
+            (((c1->group_leader != None) &&
               (c1->group_leader == c2->group_leader)) ||
              (c1->group_leader == c2->window) ||
              (c2->group_leader == c1->window)));
@@ -184,8 +182,8 @@ clientIsValidTransientOrModal (Client * c)
     {
         ScreenInfo *screen_info = c->screen_info;
         GList *index;
-        
-        /* Look for a valid transient ormodal for the same group */
+
+        /* Look for a valid transient or modal for the same group */
         for (index = screen_info->windows_stack; index; index = g_list_next (index))
         {
             Client *c2 = (Client *) index->data;
@@ -210,10 +208,10 @@ clientIsValidTransientOrModal (Client * c)
 gboolean
 clientTransientOrModalHasAncestor (Client * c, int ws)
 {
-    Client *c2 = NULL;
-    GList *index = NULL;
-    ScreenInfo *screen_info = NULL;
-    
+    Client *c2;
+    GList *index;
+    ScreenInfo *screen_info;
+
     g_return_val_if_fail (c != NULL, FALSE);
 
     TRACE ("entering clientTransientOrModalHasAncestor");
@@ -242,15 +240,17 @@ clientTransientOrModalHasAncestor (Client * c, int ws)
 Client *
 clientGetModalFor (Client * c)
 {
-    Client *latest_modal = NULL;
-    Client *c2, *c3;
     ScreenInfo *screen_info;
-    GList *modals = NULL;
+    Client *latest_modal;
+    Client *c2, *c3;
+    GList *modals;
     GList *index1, *index2;
 
     g_return_val_if_fail (c != NULL, NULL);
     TRACE ("entering clientGetModalFor");
-    
+
+    modals = NULL;
+    latest_modal = NULL;
     screen_info = c->screen_info;
     for (index1 = screen_info->windows_stack; index1; index1 = g_list_next (index1))
     {
@@ -291,12 +291,13 @@ GList *
 clientListTransient (Client * c)
 {
     ScreenInfo *screen_info;
-    GList *transients = NULL;
-    GList *index1, *index2;
     Client *c2, *c3;
+    GList *transients;
+    GList *index1, *index2;
 
     g_return_val_if_fail (c != NULL, NULL);
 
+    transients = NULL;
     screen_info = c->screen_info;
     transients = g_list_append (transients, c);
     for (index1 = screen_info->windows_stack; index1; index1 = g_list_next (index1))
@@ -331,13 +332,14 @@ GList *
 clientListTransientOrModal (Client * c)
 {
     ScreenInfo *screen_info;
-    GList *transients = NULL;
-    GList *index1, *index2;
     Client *c2, *c3;
+    GList *transients;
+    GList *index1, *index2;
 
     g_return_val_if_fail (c != NULL, NULL);
 
     screen_info = c->screen_info;
+    transients = NULL;
     transients = g_list_append (transients, c);
     for (index1 = screen_info->windows_stack; index1; index1 = g_list_next (index1))
     {
@@ -372,7 +374,7 @@ clientListTransientOrModal (Client * c)
 gboolean
 clientCheckTransientWindow (Client *c, Window w)
 {
-    GList *transients = NULL;
+    GList *transients;
     GList *index;
     Client *c2;
 
