@@ -1,22 +1,22 @@
-/*      $Id: workspaces.c 20529 2006-03-24 14:46:19Z olivier $
- 
+/*      $Id: workspaces.c 23704 2006-11-04 22:36:42Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
- 
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
  */
 
 #ifdef HAVE_CONFIG_H
@@ -29,7 +29,7 @@
 #include <glib.h>
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
-#include <libxfce4util/libxfce4util.h> 
+#include <libxfce4util/libxfce4util.h>
 
 #include "display.h"
 #include "screen.h"
@@ -45,24 +45,25 @@
 static void
 workspaceGetPosition (ScreenInfo *screen_info, int n, int * row, int * col)
 {
-    NetWmDesktopLayout * l = &screen_info->desktop_layout;
+    NetWmDesktopLayout l;
     int major_length, minor_length, tmp;
 
-    if (l->orientation == NET_WM_ORIENTATION_HORZ)
+    l = screen_info->desktop_layout;
+    if (l.orientation == NET_WM_ORIENTATION_HORZ)
     {
-        major_length = l->cols;
-        minor_length = l->rows;
+        major_length = l.cols;
+        minor_length = l.rows;
     }
     else
     {
-        major_length = l->rows;
-        minor_length = l->cols;
+        major_length = l.rows;
+        minor_length = l.cols;
     }
 
     *row = n / major_length;
     *col = n % major_length;
 
-    switch (l->start)
+    switch (l.start)
     {
         case NET_WM_TOPRIGHT:
             *col = major_length - *col - 1;
@@ -78,15 +79,15 @@ workspaceGetPosition (ScreenInfo *screen_info, int n, int * row, int * col)
             break;
     }
 
-    if (l->orientation == NET_WM_ORIENTATION_VERT)
+    if (l.orientation == NET_WM_ORIENTATION_VERT)
     {
         tmp = *row;
         *row = *col;
         *col = tmp;
-        if ((l->start == NET_WM_TOPRIGHT) || (l->start == NET_WM_BOTTOMLEFT))
+        if ((l.start == NET_WM_TOPRIGHT) || (l.start == NET_WM_BOTTOMLEFT))
         {
-            *row = l->rows - *row - 1;
-            *col = l->cols - *col - 1;
+            *row = l.rows - *row - 1;
+            *col = l.cols - *col - 1;
         }
     }
 }
@@ -94,33 +95,34 @@ workspaceGetPosition (ScreenInfo *screen_info, int n, int * row, int * col)
 static int
 workspaceGetNumber (ScreenInfo *screen_info, int row, int col)
 {
-    NetWmDesktopLayout * l = &screen_info->desktop_layout;
+    NetWmDesktopLayout l;
     int major_length, minor_length, n, tmp;
 
-    if (l->orientation == NET_WM_ORIENTATION_HORZ)
+    l = screen_info->desktop_layout;
+    if (l.orientation == NET_WM_ORIENTATION_HORZ)
     {
-        major_length = l->cols;
-        minor_length = l->rows;
+        major_length = l.cols;
+        minor_length = l.rows;
     }
     else
     {
-        major_length = l->rows;
-        minor_length = l->cols;
+        major_length = l.rows;
+        minor_length = l.cols;
     }
 
-    if (l->orientation == NET_WM_ORIENTATION_VERT)
+    if (l.orientation == NET_WM_ORIENTATION_VERT)
     {
         tmp = row;
         row = col;
         col = tmp;
-        if ((l->start == NET_WM_TOPRIGHT) || (l->start == NET_WM_BOTTOMLEFT))
+        if ((l.start == NET_WM_TOPRIGHT) || (l.start == NET_WM_BOTTOMLEFT))
         {
             row = minor_length - row - 1;
             col = major_length - col - 1;
         }
     }
 
-    switch (l->start)
+    switch (l.start)
     {
         case NET_WM_TOPRIGHT:
             col = major_length - col - 1;
@@ -148,21 +150,21 @@ modify_with_wrap (int value, int by, int limit, gboolean wrap)
     if (value >= limit)
     {
         if (!wrap)
-        { 
+        {
             value = limit - 1;
         }
-        else 
+        else
         {
             value = value % limit;
         }
     }
     else if (value < 0)
     {
-        if (!wrap) 
+        if (!wrap)
         {
             value = 0;
         }
-        else 
+        else
         {
             value = (value + limit) % limit;
         }
@@ -171,9 +173,8 @@ modify_with_wrap (int value, int by, int limit, gboolean wrap)
 }
 
 /* returns TRUE if the workspace was changed, FALSE otherwise */
-
 gboolean
-workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c2)
+workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c)
 {
     int row, col, newrow, newcol, previous_ws, n;
 
@@ -182,7 +183,7 @@ workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c2)
     newcol = modify_with_wrap(col, colmod, screen_info->desktop_layout.cols, screen_info->params->wrap_layout);
     n = workspaceGetNumber(screen_info, newrow, newcol);
 
-    if (n == screen_info->current_ws) 
+    if (n == screen_info->current_ws)
     {
         return FALSE;
     }
@@ -190,7 +191,7 @@ workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c2)
     previous_ws = screen_info->current_ws;
     if ((n >= 0) && (n < screen_info->workspace_count))
     {
-        workspaceSwitch(screen_info, n, c2);
+        workspaceSwitch(screen_info, n, c, TRUE);
     }
     else if (screen_info->params->wrap_layout)
     {
@@ -200,36 +201,36 @@ workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c2)
         }
         else
         {
-            if (colmod > 0) 
+            if (colmod > 0)
             {
                 newcol = 0;
             }
-            else if (rowmod > 0) 
+            else if (rowmod > 0)
             {
                 newrow = 0;
             }
-            else if (rowmod < 0) 
+            else if (rowmod < 0)
             {
                 newrow--;
             }
-            else 
+            else
             {
                 return FALSE;
             }
 
             n = workspaceGetNumber(screen_info, newrow, newcol);
         }
-        workspaceSwitch(screen_info, n, c2);
+        workspaceSwitch(screen_info, n, c, TRUE);
     }
 
     return (screen_info->current_ws != previous_ws);
 }
 
 void
-workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2)
+workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2, gboolean update_focus)
 {
-    DisplayInfo *display_info = screen_info->display_info;
-    Client *c, *new_focus = NULL;
+    DisplayInfo *display_info;
+    Client *c, *new_focus;
     Client *previous;
     GList *index;
     GList *list_hide;
@@ -237,9 +238,10 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2)
     int rx, ry, wx, wy;
     unsigned int mask;
     unsigned long data[1];
-    
+
     TRACE ("entering workspaceSwitch");
 
+    display_info = screen_info->display_info;
     if ((new_ws == screen_info->current_ws) && (screen_info->params->toggle_workspaces))
     {
         new_ws = screen_info->previous_ws;
@@ -270,71 +272,86 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2)
 
     screen_info->previous_ws = screen_info->current_ws;
     screen_info->current_ws = new_ws;
+
+    new_focus = NULL;
+    list_hide = NULL;
+    previous  = NULL;
+    c = clientGetFocus ();
+
     if (c2)
     {
         clientSetWorkspace (c2, new_ws, FALSE);
     }
 
-    list_hide = NULL;
-    previous = clientGetFocus ();
+    if (c)
+    {
+        if (c->type & WINDOW_REGULAR_FOCUSABLE)
+        {
+            previous = c;
+        }
+        if (c2 == c)
+        {
+            new_focus = c2;
+        }
+    }
 
-    /* First pass */
+    /* First pass: Show, from top to bottom */
+    for (index = g_list_last(screen_info->windows_stack); index; index = g_list_previous (index))
+    {
+        c = (Client *) index->data;
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
+        {
+            clientSetWorkspace (c, new_ws, TRUE);
+        }
+        else if (c->win_workspace == new_ws)
+        {
+            if (!FLAG_TEST (c->flags, CLIENT_FLAG_ICONIFIED))
+            {
+                if (!clientIsTransientOrModal (c))
+                {
+                    clientShow (c, FALSE);
+                }
+            }
+        }
+    }
+
+    /* Second pass: Hide from bottom to top */
     for (index = screen_info->windows_stack; index; index = g_list_next (index))
     {
         c = (Client *) index->data;
-        if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE)
-            && !FLAG_TEST (c->flags, CLIENT_FLAG_STICKY) 
-            && ((c->win_workspace != new_ws)))
+        
+        if (c->win_workspace != new_ws)
         {
             if (c == previous)
             {
                 FLAG_SET (previous->xfwm_flags, XFWM_FLAG_FOCUS);
                 clientSetFocus (screen_info, NULL, myDisplayGetCurrentTime (display_info), FOCUS_IGNORE_MODAL);
             }
-            if (!clientIsValidTransientOrModal (c))
+            if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE) && !FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
             {
-                /* Just build of windows that will be hidden, so that
-                   we can record the previous focus even when on a
-                   transient (otherwise the transient vanishes along
-                   with its parent window which is placed below...
-                 */
-                list_hide = g_list_append (list_hide, c);
+                if (!clientIsTransientOrModal (c))
+                {
+                    clientHide (c, new_ws, FALSE);
+                }
             }
         }
     }
 
-    /* First pass and a half :) */
-    if (list_hide)
-    {
-        for (index = list_hide; index; index = g_list_next (index))
-        {
-            c = (Client *) index->data;
-            clientHide (c, new_ws, FALSE);
-        }
-        g_list_free (list_hide);
-    }
-
-    /* Second pass */
+    /* Third pass: Check for focus, from top to bottom */
     for (index = g_list_last(screen_info->windows_stack); index; index = g_list_previous (index))
     {
         c = (Client *) index->data;
-        if (FLAG_TEST (c->flags, CLIENT_FLAG_STICKY)
-            || FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE))
+        
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
         {
-            clientSetWorkspace (c, new_ws, TRUE);
-            if (c == previous)
+            if ((!new_focus) && (c == previous) && (c->type & WINDOW_REGULAR_FOCUSABLE))
             {
                 new_focus = c;
             }
             FLAG_UNSET (c->xfwm_flags, XFWM_FLAG_FOCUS);
         }
-        else if ((c->win_workspace == new_ws)
-            && !FLAG_TEST (c->flags, CLIENT_FLAG_ICONIFIED))
+        else if (c->win_workspace == new_ws)
         {
-            if (!clientIsValidTransientOrModal (c))
-            {
-                clientShow (c, FALSE);
-            }
             if ((!new_focus) && FLAG_TEST (c->xfwm_flags, XFWM_FLAG_FOCUS))
             {
                 new_focus = c;
@@ -345,7 +362,7 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2)
 
     setHint (display_info, screen_info->xroot, WIN_WORKSPACE, new_ws);
     data[0] = new_ws;
-    XChangeProperty (myScreenGetXDisplay (screen_info), screen_info->xroot, 
+    XChangeProperty (myScreenGetXDisplay (screen_info), screen_info->xroot,
                      display_info->atoms[NET_CURRENT_DESKTOP], XA_CARDINAL, 32,
                      PropModeReplace, (unsigned char *) data, 1);
     if (!(screen_info->params->click_to_focus))
@@ -360,23 +377,23 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2)
         }
     }
 
-    myScreenUngrabPointer (screen_info, myDisplayGetCurrentTime (display_info));
+    if (update_focus)
+    {
+        if (new_focus)
+        {
+            clientSetFocus (screen_info, new_focus, myDisplayGetCurrentTime (display_info), NO_FOCUS_FLAG);
+        }
+        else
+        {
+            clientFocusTop (screen_info, WIN_LAYER_NORMAL);
+        }
+    }
 
-    if (new_focus)
-    {
-        clientSetFocus (new_focus->screen_info, new_focus, myDisplayGetCurrentTime (display_info), NO_FOCUS_FLAG);
-    }
-#if 0
-    else
-    {
-        /* If we can't get a window to focus, just pick the one on top */
-        clientFocusTop (screen_info, WIN_LAYER_ABOVE_DOCK);
-    }
-#endif
+    myScreenUngrabPointer (screen_info, myDisplayGetCurrentTime (display_info));
 }
 
 void
-workspaceSetNames (ScreenInfo * screen_info, char **names, int items)
+workspaceSetNames (ScreenInfo * screen_info, gchar **names, int items)
 {
     if (screen_info->workspace_names)
     {
@@ -390,7 +407,7 @@ workspaceSetNames (ScreenInfo * screen_info, char **names, int items)
 void
 workspaceSetCount (ScreenInfo * screen_info, int count)
 {
-    DisplayInfo *display_info = NULL;
+    DisplayInfo *display_info;
     Client *c;
     int i;
 
@@ -419,21 +436,19 @@ workspaceSetCount (ScreenInfo * screen_info, int count)
     }
     if (screen_info->current_ws > count - 1)
     {
-        workspaceSwitch (screen_info, count - 1, NULL);
+        workspaceSwitch (screen_info, count - 1, NULL, TRUE);
     }
-    setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count, 
-                    gdk_screen_get_width (screen_info->gscr),
-                    gdk_screen_get_height (screen_info->gscr),
-                    screen_info->margins);
+    setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count,
+                    screen_info->width, screen_info->height, screen_info->margins);
     /* Recompute the layout based on the (changed) number of desktops */
-    getDesktopLayout(display_info, screen_info->xroot, screen_info->workspace_count, 
+    getDesktopLayout(display_info, screen_info->xroot, screen_info->workspace_count,
                      &screen_info->desktop_layout);
 }
 
 void
 workspaceUpdateArea (ScreenInfo *screen_info)
 {
-    DisplayInfo *display_info = NULL;
+    DisplayInfo *display_info;
     Client *c;
     int prev_top;
     int prev_left;
@@ -448,10 +463,10 @@ workspaceUpdateArea (ScreenInfo *screen_info)
     TRACE ("entering workspaceUpdateArea");
 
     display_info = screen_info->display_info;
-    prev_top = screen_info->margins[TOP];
-    prev_left = screen_info->margins[LEFT];
-    prev_right = screen_info->margins[RIGHT];
-    prev_bottom = screen_info->margins[BOTTOM];
+    prev_top = screen_info->margins[STRUTS_TOP];
+    prev_left = screen_info->margins[STRUTS_LEFT];
+    prev_right = screen_info->margins[STRUTS_RIGHT];
+    prev_bottom = screen_info->margins[STRUTS_BOTTOM];
 
     for (i = 0; i < 4; i++)
     {
@@ -463,21 +478,21 @@ workspaceUpdateArea (ScreenInfo *screen_info)
         if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_STRUT)
             && FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE))
         {
-            screen_info->margins[TOP]    = MAX (screen_info->margins[TOP],    c->struts[TOP]);
-            screen_info->margins[LEFT]   = MAX (screen_info->margins[LEFT],   c->struts[LEFT]);
-            screen_info->margins[RIGHT]  = MAX (screen_info->margins[RIGHT],  c->struts[RIGHT]);
-            screen_info->margins[BOTTOM] = MAX (screen_info->margins[BOTTOM], c->struts[BOTTOM]);
+            screen_info->margins[STRUTS_TOP]    = MAX (screen_info->margins[STRUTS_TOP],    c->struts[STRUTS_TOP]);
+            screen_info->margins[STRUTS_LEFT]   = MAX (screen_info->margins[STRUTS_LEFT],   c->struts[STRUTS_LEFT]);
+            screen_info->margins[STRUTS_RIGHT]  = MAX (screen_info->margins[STRUTS_RIGHT],  c->struts[STRUTS_RIGHT]);
+            screen_info->margins[STRUTS_BOTTOM] = MAX (screen_info->margins[STRUTS_BOTTOM], c->struts[STRUTS_BOTTOM]);
         }
     }
 
-    if ((prev_top != screen_info->margins[TOP]) || (prev_left != screen_info->margins[LEFT])
-        || (prev_right != screen_info->margins[RIGHT])
-        || (prev_bottom != screen_info->margins[BOTTOM]))
+    if ((prev_top != screen_info->margins[STRUTS_TOP]) || (prev_left != screen_info->margins[STRUTS_LEFT])
+        || (prev_right != screen_info->margins[STRUTS_RIGHT])
+        || (prev_bottom != screen_info->margins[STRUTS_BOTTOM]))
     {
         TRACE ("Margins have changed, updating net_workarea");
-        setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count, 
-                        gdk_screen_get_width (screen_info->gscr),
-                        gdk_screen_get_height (screen_info->gscr),
-                        screen_info->margins);
+        setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count,
+                        screen_info->width, screen_info->height, screen_info->margins);
+        /* Also prevent windows from being off screen, just like when screen is resized */
+        clientScreenResize(screen_info);
     }
 }
